from Carbon import Dlg, Evt, Events, Fm
from Carbon import Menu, Qd, Win, Windows
import FrameWork
import Wbase
import MacOS
import struct
import traceback
from types import InstanceType, StringType

if hasattr(Win, "FrontNonFloatingWindow"):
    MyFrontWindow = Win.FrontNonFloatingWindow
else:
    MyFrontWindow = Win.FrontWindow


class Window(FrameWork.Window, Wbase.SelectableWidget):

    windowkind = Windows.documentProc

    def __init__(self, possize, title="", minsize=None, maxsize=None,
                    tabbable=1, show=1, fontsettings=None):
        import W
        if fontsettings is None:
            fontsettings = W.getdefaultfont()
        self._fontsettings = fontsettings
        W.SelectableWidget.__init__(self, possize)
        self._globalbounds = l, t, r, b = self.getwindowbounds(possize, minsize)
        self._bounds = (0, 0, r - l, b - t)
        self._tabchain = []
        self._currentwidget = None
        self.title = title
        self._parentwindow = self
        self._tabbable = tabbable
        self._defaultbutton = None
        self._drawwidgetbounds = 0
        self._show = show
        self._lastrollover = None
        self.hasclosebox = 1
        # XXX the following is not really compatible with the
        #  new (system >= 7.5) window procs.
        if minsize:
            self._hasgrowbox = 1
            self.windowkind = self.windowkind | 8
            l, t = minsize
            if maxsize:
                r, b = maxsize[0] + 1, maxsize[1] + 1
            else:
                r, b = 32000, 32000
            self.growlimit = (l, t, r, b)
        else:
            self._hasgrowbox = 0
            if (self.windowkind == 0 or self.windowkind >= 8) and self.windowkind < 1000:
                self.windowkind = self.windowkind | 4
        FrameWork.Window.__init__(self, W.getapplication())

    def gettitle(self):
        return self.title

    def settitle(self, title):
        self.title = title
        if self.wid:
            self.wid.SetWTitle(title)

    def getwindowbounds(self, size, minsize = None):
        return windowbounds(size, minsize)

    def getcurrentwidget(self):
        return self._currentwidget

    def show(self, onoff):
        if onoff:
            self.wid.ShowWindow()
        else:
            self.wid.HideWindow()

    def isvisible(self):
        return self.wid.IsWindowVisible()

    def select(self):
        self.wid.SelectWindow()
        # not sure if this is the best place, I need it when
        # an editor gets selected, and immediately scrolled
        # to a certain line, waste scroll assumes everything
        # to be in tact.
        self.do_rawupdate(self.wid, "DummyEvent")

    def open(self):
        self.wid = Win.NewCWindow(self._globalbounds, self.title, self._show,
                self.windowkind, -1, self.hasclosebox, 0)
        self.SetPort()
        fontname, fontstyle, fontsize, fontcolor = self._fontsettings
        fnum = Fm.GetFNum(fontname)
        if fnum == 0:
            fnum = Fm.GetFNum("Geneva")
        Qd.TextFont(fnum)
        Qd.TextFace(fontstyle)
        Qd.TextSize(fontsize)
        if self._bindings.has_key("<open>"):
            callback = self._bindings["<open>"]
            callback()
        for w in self._widgets:
            w.forall_frombottom("open")
        self._maketabchain()
        if self._tabbable:
            self.bind('tab', self.nextwidget)
            self.bind('shifttab', self.previouswidget)
        else:
            self._hasselframes = 0
        if self._tabchain:
            self._tabchain[0].select(1)
        self.do_postopen()

    def close(self):
        if not self.wid:
            return  # we are already closed
        if self._bindings.has_key("<close>"):
            callback = self._bindings["<close>"]
            try:
                rv = callback()
            except:
                print 'error in <close> callback'
                traceback.print_exc()
            else:
                if rv:
                    return rv
        #for key in self._widgetsdict.keys():
        #       self._removewidget(key)
        self.forall_butself("close")
        Wbase.SelectableWidget.close(self)
        self._tabchain = []
        self._currentwidget = None
        self.wid.HideWindow()
        self.do_postclose()

    def domenu_close(self, *args):
        self.close()

    def getbounds(self):
        return self._globalbounds

    def setbounds(self, bounds):
        l, t, r, b = bounds
        self.move(l, t)
        self.resize(r-l, b-t)

    def move(self, x, y = None):
        """absolute move"""
        if y == None:
            x, y = x
        self.wid.MoveWindow(x, y, 0)

    def resize(self, x, y = None):
        if not self._hasgrowbox:
            return  # hands off!
        if y == None:
            x, y = x
        self.SetPort()
        self.GetWindow().InvalWindowRect(self.getgrowrect())
        self.wid.SizeWindow(x, y, 1)
        self._calcbounds()

    def test(self, point):
        return 1

    def draw(self, visRgn = None):
        if self._hasgrowbox:
            self.tempcliprect(self.getgrowrect())
            self.wid.DrawGrowIcon()
            self.restoreclip()

    def idle(self, *args):
        self.SetPort()
        point = Evt.GetMouse()
        widget = self.findwidget(point, 0)
        if self._bindings.has_key("<idle>"):
            callback = self._bindings["<idle>"]
            if callback():
                return
        if self._currentwidget is not None and hasattr(self._currentwidget, "idle"):
            if self._currentwidget._bindings.has_key("<idle>"):
                callback = self._currentwidget._bindings["<idle>"]
                if callback():
                    return
            if self._currentwidget.idle():
                return
        if widget is not None and hasattr(widget, "rollover"):
            if 1:   #self._lastrollover <> widget:
                if self._lastrollover:
                    self._lastrollover.rollover(point, 0)
                self._lastrollover = widget
                self._lastrollover.rollover(point, 1)
        else:
            if self._lastrollover:
                self._lastrollover.rollover(point, 0)
            self._lastrollover = None
            Wbase.SetCursor("arrow")

    def xxx___select(self, widget):
        if self._currentwidget == widget:
            return
        if self._bindings.has_key("<select>"):
            callback = self._bindings["<select>"]
            if callback(widget):
                return
        if widget is None:
            if self._currentwidget is not None:
                self._currentwidget.select(0)
        elif type(widget) == InstanceType and widget._selectable:
            widget.select(1)
        elif widget == -1 or widget == 1:
            if len(self._tabchain) <= 1:
                return
            temp = self._tabchain[(self._tabchain.index(self._currentwidget) + widget) % len(self._tabchain)]
            temp.select(1)
        else:
            raise TypeError, "Widget is not selectable"

    def setdefaultbutton(self, newdefaultbutton = None, *keys):
        if newdefaultbutton == self._defaultbutton:
            return
        if self._defaultbutton:
            self._defaultbutton._setdefault(0)
        if not newdefaultbutton:
            self.bind("return", None)
            self.bind("enter", None)
            return
        import Wcontrols
        if not isinstance(newdefaultbutton, Wcontrols.Button):
            raise TypeError, "widget is not a button"
        self._defaultbutton = newdefaultbutton
        self._defaultbutton._setdefault(1)
        if not keys:
            self.bind("return", self._defaultbutton.push)
            self.bind("enter", self._defaultbutton.push)
        else:
            for key in keys:
                self.bind(key, self._defaultbutton.push)

    def nextwidget(self):
        self.xxx___select(1)

    def previouswidget(self):
        self.xxx___select(-1)

    def drawwidgetbounds(self, onoff):
        self._drawwidgetbounds = onoff
        self.SetPort()
        self.GetWindow().InvalWindowRect(self._bounds)

    def _drawbounds(self):
        pass

    def _maketabchain(self):
        # XXX This has to change, it's no good when we are adding or deleting widgets.
        # XXX Perhaps we shouldn't keep a "tabchain" at all.
        self._hasselframes = 0
        self._collectselectablewidgets(self._widgets)
        if self._hasselframes and len(self._tabchain) > 1:
            self._hasselframes = 1
        else:
            self._hasselframes = 0

    def _collectselectablewidgets(self, widgets):
        import W
        for w in widgets:
            if w._selectable:
                self._tabchain.append(w)
                if isinstance(w, W.List):
                    self._hasselframes = 1
            self._collectselectablewidgets(w._widgets)

    def _calcbounds(self):
        self._possize = self.wid.GetWindowPort().GetPortBounds()[2:]
        w, h = self._possize
        self._bounds = (0, 0, w, h)
        self.wid.GetWindowContentRgn(scratchRegion)
        l, t, r, b = GetRgnBounds(scratchRegion)
        self._globalbounds = l, t, l + w, t + h
        for w in self._widgets:
            w._calcbounds()

    # FrameWork override methods
    def do_inDrag(self, partcode, window, event):
        where = event[3]
        self.wid.GetWindowContentRgn(scratchRegion)
        was_l, was_t, r, b = GetRgnBounds(scratchRegion)
        window.DragWindow(where, self.draglimit)
        self.wid.GetWindowContentRgn(scratchRegion)
        is_l, is_t, r, b = GetRgnBounds(scratchRegion)
        self._globalbounds = Qd.OffsetRect(self._globalbounds,
                                is_l - was_l, is_t - was_t)

    def do_char(self, char, event):
        import Wkeys
        (what, message, when, where, modifiers) = event
        key = char
        if Wkeys.keynames.has_key(key):
            key = Wkeys.keynames[key]
        if modifiers & Events.shiftKey:
            key = 'shift' + key
        if modifiers & Events.cmdKey:
            key = 'cmd' + key
        if modifiers & Events.controlKey:
            key = 'control' + key
        if self._bindings.has_key("<key>"):
            callback = self._bindings["<key>"]
            if Wbase.CallbackCall(callback, 0, char, event):
                return
        if self._bindings.has_key(key):
            callback = self._bindings[key]
            Wbase.CallbackCall(callback, 0, char, event)
        elif self._currentwidget is not None:
            if self._currentwidget._bindings.has_key(key):
                callback = self._currentwidget._bindings[key]
                Wbase.CallbackCall(callback, 0, char, event)
            else:
                if self._currentwidget._bindings.has_key("<key>"):
                    callback = self._currentwidget._bindings["<key>"]
                    if Wbase.CallbackCall(callback, 0, char, event):
                        return
                self._currentwidget.key(char, event)

    def do_contentclick(self, point, modifiers, event):
        widget = self.findwidget(point)
        if widget is not None:
            if self._bindings.has_key("<click>"):
                callback = self._bindings["<click>"]
                if Wbase.CallbackCall(callback, 0, point, modifiers):
                    return
            if widget._bindings.has_key("<click>"):
                callback = widget._bindings["<click>"]
                if Wbase.CallbackCall(callback, 0, point, modifiers):
                    return
            if widget._selectable:
                widget.select(1, 1)
            widget.click(point, modifiers)

    def do_update(self, window, event):
        Qd.EraseRgn(window.GetWindowPort().visRgn)
        self.forall_frombottom("draw", window.GetWindowPort().visRgn)
        if self._drawwidgetbounds:
            self.forall_frombottom("_drawbounds")

    def do_activate(self, onoff, event):
        if not onoff:
            if self._lastrollover:
                self._lastrollover.rollover((0, 0), 0)
                self._lastrollover = None
        self.SetPort()
        self.forall("activate", onoff)
        self.draw()

    def do_postresize(self, width, height, window):
        self.GetWindow().InvalWindowRect(self.getgrowrect())
        self._calcbounds()

    def do_inGoAway(self, partcode, window, event):
        where = event[3]
        closeall = event[4] & Events.optionKey
        if window.TrackGoAway(where):
            if not closeall:
                self.close()
            else:
                for window in self.parent._windows.values():
                    rv = window.close()
                    if rv and rv > 0:
                        return

    # utilities
    def tempcliprect(self, tempcliprect):
        tempclip = Qd.NewRgn()
        Qd.RectRgn(tempclip, tempcliprect)
        self.tempclip(tempclip)
        Qd.DisposeRgn(tempclip)

    def tempclip(self, tempclip):
        if not hasattr(self, "saveclip"):
            self.saveclip = []
        saveclip = Qd.NewRgn()
        Qd.GetClip(saveclip)
        self.saveclip.append(saveclip)
        Qd.SetClip(tempclip)

    def restoreclip(self):
        Qd.SetClip(self.saveclip[-1])
        Qd.DisposeRgn(self.saveclip[-1])
        del self.saveclip[-1]

    def getgrowrect(self):
        l, t, r, b = self.wid.GetWindowPort().GetPortBounds()
        return (r - 15, b - 15, r, b)

    def has_key(self, key):
        return self._widgetsdict.has_key(key)

    def __getattr__(self, attr):
        global _successcount, _failcount, _magiccount
        if self._widgetsdict.has_key(attr):
            _successcount = _successcount + 1
            return self._widgetsdict[attr]
        if self._currentwidget is None or (attr[:7] <> 'domenu_' and
                        attr[:4] <> 'can_' and attr <> 'insert'):
            _failcount = _failcount + 1
            raise AttributeError, attr
        # special case: if a domenu_xxx, can_xxx or insert method is asked for,
        # see if the active widget supports it
        _magiccount = _magiccount + 1
        return getattr(self._currentwidget, attr)

_successcount = 0
_failcount = 0
_magiccount = 0

class Dialog(Window):

    windowkind = Windows.movableDBoxProc

    # this __init__ seems redundant, but it's not: it has less args
    def __init__(self, possize, title = ""):
        Window.__init__(self, possize, title)

    def can_close(self, *args):
        return 0

    def getwindowbounds(self, size, minsize = None):
        screenbounds = sl, st, sr, sb = Qd.GetQDGlobalsScreenBits().bounds
        w, h = size
        l = sl + (sr - sl - w) / 2
        t = st + (sb - st - h) / 3
        return l, t, l + w, t + h


class ModalDialog(Dialog):

    def __init__(self, possize, title = ""):
        Dialog.__init__(self, possize, title)
        if title:
            self.windowkind = Windows.movableDBoxProc
        else:
            self.windowkind = Windows.dBoxProc

    def open(self):
        import W
        Dialog.open(self)
        self.app = W.getapplication()
        self.done = 0
        Menu.HiliteMenu(0)
        app = self.parent
        app.enablemenubar(0)
        try:
            self.mainloop()
        finally:
            app.enablemenubar(1)

    def close(self):
        if not self.wid:
            return  # we are already closed
        self.done = 1
        del self.app
        Dialog.close(self)

    def mainloop(self):
        if hasattr(MacOS, 'EnableAppswitch'):
            saveyield = MacOS.EnableAppswitch(-1)
        while not self.done:
            #self.do1event()
            self.do1event(  Events.keyDownMask +
                                    Events.autoKeyMask +
                                    Events.activMask +
                                    Events.updateMask +
                                    Events.mDownMask +
                                    Events.mUpMask,
                                    10)
        if hasattr(MacOS, 'EnableAppswitch'):
            MacOS.EnableAppswitch(saveyield)

    def do1event(self, mask = Events.everyEvent, wait = 0):
        ok, event = self.app.getevent(mask, wait)
        if Dlg.IsDialogEvent(event):
            if self.app.do_dialogevent(event):
                return
        if ok:
            self.dispatch(event)
        else:
            self.app.idle(event)

    def do_keyDown(self, event):
        self.do_key(event)

    def do_autoKey(self, event):
        if not event[-1] & Events.cmdKey:
            self.do_key(event)

    def do_key(self, event):
        (what, message, when, where, modifiers) = event
        #w = Win.FrontWindow()
        #if w <> self.wid:
        #       return
        c = chr(message & Events.charCodeMask)
        if modifiers & Events.cmdKey:
            self.app.checkmenus(self)
            result = Menu.MenuKey(ord(c))
            id = (result>>16) & 0xffff      # Hi word
            item = result & 0xffff          # Lo word
            if id:
                self.app.do_rawmenu(id, item, None, event)
                return
        self.do_char(c, event)

    def do_mouseDown(self, event):
        (what, message, when, where, modifiers) = event
        partcode, wid = Win.FindWindow(where)
        #
        # Find the correct name.
        #
        if FrameWork.partname.has_key(partcode):
            name = "do_" + FrameWork.partname[partcode]
        else:
            name = "do_%d" % partcode

        if name == "do_inDesk":
            if hasattr(MacOS, "HandleEvent"):
                MacOS.HandleEvent(event)
            else:
                print 'Unexpected inDesk event:', event
            return
        if wid == self.wid:
            try:
                handler = getattr(self, name)
            except AttributeError:
                handler = self.app.do_unknownpartcode
        else:
            #MacOS.HandleEvent(event)
            if name == 'do_inMenuBar':
                handler = getattr(self.parent, name)
            else:
                return
        handler(partcode, wid, event)

    def dispatch(self, event):
        (what, message, when, where, modifiers) = event
        if FrameWork.eventname.has_key(what):
            name = "do_" + FrameWork.eventname[what]
        else:
            name = "do_%d" % what
        try:
            handler = getattr(self, name)
        except AttributeError:
            try:
                handler = getattr(self.app, name)
            except AttributeError:
                handler = self.app.do_unknownevent
        handler(event)


def FrontWindowInsert(stuff):
    if not stuff:
        return
    if type(stuff) <> StringType:
        raise TypeError, 'string expected'
    import W
    app = W.getapplication()
    wid = MyFrontWindow()
    if wid and app._windows.has_key(wid):
        window = app._windows[wid]
        if hasattr(window, "insert"):
            try:
                window.insert(stuff)
                return
            except:
                pass
    import EasyDialogs
    if EasyDialogs.AskYesNoCancel(
                    "Can't find window or widget to insert text into; copy to clipboard instead?",
                    1) == 1:
        from Carbon import Scrap
        if hasattr(Scrap, 'PutScrap'):
            Scrap.ZeroScrap()
            Scrap.PutScrap('TEXT', stuff)
        else:
            Scrap.ClearCurrentScrap()
            sc = Scrap.GetCurrentScrap()
            sc.PutScrapFlavor('TEXT', 0, stuff)


# not quite based on the same function in FrameWork
_windowcounter = 0

def getnextwindowpos():
    global _windowcounter
    rows = 8
    l = 4 * (rows + 1 - (_windowcounter % rows) + _windowcounter / rows)
    t = 44 + 20 * (_windowcounter % rows)
    _windowcounter = _windowcounter + 1
    return l, t

def windowbounds(preferredsize, minsize=None):
    "Return sensible window bounds"

    global _windowcounter
    if len(preferredsize) == 4:
        bounds = l, t, r, b = preferredsize
        desktopRgn = Win.GetGrayRgn()
        tempRgn = Qd.NewRgn()
        Qd.RectRgn(tempRgn, bounds)
        union = Qd.UnionRgn(tempRgn, desktopRgn, tempRgn)
        equal = Qd.EqualRgn(tempRgn, desktopRgn)
        Qd.DisposeRgn(tempRgn)
        if equal:
            return bounds
        else:
            preferredsize = r - l, b - t
    if not minsize:
        minsize = preferredsize
    minwidth, minheight = minsize
    width, height = preferredsize

    sl, st, sr, sb = screenbounds = Qd.InsetRect(Qd.GetQDGlobalsScreenBits().bounds, 4, 4)
    l, t = getnextwindowpos()
    if (l + width) > sr:
        _windowcounter = 0
        l, t = getnextwindowpos()
    r = l + width
    b = t + height
    if (t + height) > sb:
        b = sb
        if (b - t) < minheight:
            b = t + minheight
    return l, t, r, b

scratchRegion = Qd.NewRgn()

# util -- move somewhere convenient???
def GetRgnBounds(the_Rgn):
    (t, l, b, r) = struct.unpack("hhhh", the_Rgn.data[2:10])
    return (l, t, r, b)



"""Async sockets, build on top of Sam Rushing's excellent async library"""

import asyncore
import socket
from socket import AF_INET, SOCK_STREAM
import string
import cStringIO
import mimetools
import httplib


__version__ = "0.9"
__author__ = "jvr"

BUFSIZE = 512

VERBOSE = 1

class Server(asyncore.dispatcher):

    """Generic asynchronous server class"""

    def __init__(self, port, handler_class, backlog=1, host=""):
        """arguments:
        - port: the port to listen to
        - handler_class: class to handle requests
        - backlog: backlog queue size (optional) (don't fully understand, see socket docs)
        - host: host name (optional: can be empty to use default host name)
        """
        if VERBOSE:
            print "Starting", self.__class__.__name__
        self.handler_class = handler_class
        asyncore.dispatcher.__init__(self)
        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
        self.bind((host, port))
        self.listen(backlog)

    def handle_accept(self):
        conn, addr = self.accept()
        if VERBOSE:
            print 'Incoming Connection from %s:%d' % addr
        self.handler_class(conn)


class ProxyServer(Server):

    """Generic proxy server class"""

    def __init__(self, port, handler_class, proxyaddr=None, closepartners=0):
        """arguments:
        - port: the port to listen to
        - handler_class: proxy class to handle requests
        - proxyaddr: a tuple containing the address and
          port of a remote host to connect to (optional)
        - closepartners: boolean, specifies whether we should close
          all proxy connections or not (optional). http seems to *not*
          want this, but telnet does...
        """
        Server.__init__(self, port, handler_class, 1, "")
        self.proxyaddr = proxyaddr
        self.closepartners = closepartners

    def handle_accept(self):
        conn, addr = self.accept()
        if VERBOSE:
            print 'Incoming Connection from %s:%d' % addr
        self.handler_class(conn, self.proxyaddr, closepartner=self.closepartners)


class Connection(asyncore.dispatcher):

    """Generic connection class"""

    def __init__(self, sock_or_address=None, readfunc=None, terminator=None):
        """arguments:
        - sock_or_address: either a socket, or a tuple containing the name
        and port number of a remote host
        - readfunc: callback function (optional). Will be called whenever
          there is some data available, or when an appropraite terminator
          is found.
        - terminator: string which specifies when a read is complete (optional)"""
        self._out_buffer = ""
        self._in_buffer = ""
        self.readfunc = readfunc
        self.terminator = terminator
        asyncore.dispatcher.__init__(self)
        if hasattr(sock_or_address, "fileno"):
            self.set_socket(sock_or_address)
        else:
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            sock.setblocking(0)
            self.set_socket(sock)
            if sock_or_address:
                self.connect(sock_or_address)

    # public methods
    def send(self, data):
        self._out_buffer = self._out_buffer + data

    def recv(self, bytes=-1):
        if bytes == -1:
            bytes = len(self._in_buffer)
        data = self._in_buffer[:bytes]
        self._in_buffer = self._in_buffer[bytes:]
        return data

    def set_terminator(self, terminator):
        self.terminator = terminator

    # override this if you want to control the incoming data stream
    def handle_incoming_data(self, data):
        if self.readfunc:
            if self.terminator:
                self._in_buffer = self._in_buffer + data
                pos = string.find(self._in_buffer, self.terminator)
                if pos < 0:
                    return
                data = self._in_buffer[:pos]
                self._in_buffer = self._in_buffer[pos + len(self.terminator):]
                self.readfunc(data)
            else:
                self.readfunc(self._in_buffer + data)
                self._in_buffer = ""
        else:
            self._in_buffer = self._in_buffer + data

    # internal muck
    def handle_read(self):
        data = asyncore.dispatcher.recv(self, BUFSIZE)
        if data:
            if VERBOSE > 2:
                print "incoming -> %x %r" % (id(self), data)
            self.handle_incoming_data(data)

    def handle_write(self):
        if self._out_buffer:
            sent = self.socket.send(self._out_buffer[:BUFSIZE])
            if VERBOSE > 2:
                print "outgoing -> %x %r" % (id(self), self._out_buffer[:sent])
            self._out_buffer = self._out_buffer[sent:]

    def close(self):
        if self.readfunc and self._in_buffer:
            self.readfunc(self._in_buffer)
            self._in_buffer = ""
        #elif VERBOSE > 1 and self._in_buffer:
        #       print "--- there is unread data: %r", (self._in_buffer,)
        asyncore.dispatcher.close(self)

    def handle_close(self):
        self.close()

    def handle_connect(self):
        pass


class ConnectionUI:

    """Glue to let a connection tell things to the UI in a standardized way.

    The protocoll defines four functions, which the connection will call:

            def settotal(int total): gets called when the connection knows the data size
            def setcurrent(int current): gets called when some new data has arrived
            def done(): gets called when the transaction is complete
            def error(type, value, tb): gets called wheneven an error occured
    """

    def __init__(self, settotal_func, setcurrent_func, done_func, error_func):
        self.settotal = settotal_func
        self.setcurrent = setcurrent_func
        self.done = done_func
        self.error = error_func


class HTTPError(socket.error): pass


class HTTPClient(Connection, httplib.HTTP):

    """Asynchronous HTTP connection"""

    def __init__(self, (host, port), datahandler, ui=None):
        Connection.__init__(self, (host, port))
        self.datahandler = datahandler
        self.ui = ui
        self.buf = ""
        self.doneheaders = 0
        self.done = 0
        self.headers = None
        self.length = None
        self.pos = 0

    def getreply(self):
        raise TypeError, "getreply() is not supported in async HTTP connection"

    def handle_incoming_data(self, data):
        assert not self.done
        if not self.doneheaders:
            self.buf = self.buf + data
            pos = string.find(self.buf, "\r\n\r\n")
            if pos >= 0:
                self.handle_reply(self.buf[:pos+4])
                length = self.headers.getheader("Content-Length")
                if length is not None:
                    self.length = int(length)
                    if self.ui is not None:
                        self.ui.settotal(self.length)
                else:
                    self.length = -1
                self.doneheaders = 1
                self.handle_data(self.buf[pos+4:])
                self.buf = ""
        else:
            self.handle_data(data)

    def handle_reply(self, data):
        f = cStringIO.StringIO(data)
        ver, code, msg = string.split(f.readline(), None, 2)
        code = int(code)
        msg = string.strip(msg)
        if code <> 200:
            # Hm, this is what *I* need, but probably not correct...
            raise HTTPError, (code, msg)
        self.headers = mimetools.Message(f)

    def handle_data(self, data):
        self.pos = self.pos + len(data)
        if self.ui is not None:
            self.ui.setcurrent(self.pos)
        self.datahandler(data)
        if self.pos >= self.length:
            self.datahandler("")
            self.done = 1
            if self.ui is not None:
                self.ui.done()

    def handle_error(self, type, value, tb):
        if self.ui is not None:
            self.ui.error(type, value, tb)
        else:
            Connection.handle_error(self, type, value, tb)

    def log(self, message):
        if VERBOSE:
            print 'LOG:', message


class PyMessage:

    def __init__(self):
        self._buf = ""
        self._len = None
        self._checksum = None

    def feed(self, data):
        self._buf = self._buf + data
        if self._len is None:
            if len(self._buf) >= 8:
                import struct
                self._len, self._checksum = struct.unpack("ll", self._buf[:8])
                self._buf = self._buf[8:]
        if self._len is not None:
            if len(self._buf) >= self._len:
                import zlib
                data = self._buf[:self._len]
                leftover = self._buf[self._len:]
                self._buf = None
                assert self._checksum == zlib.adler32(data), "corrupt data"
                self.data = data
                return 1, leftover
            else:
                return 0, None
        else:
            return 0, None


class PyConnection(Connection):

    def __init__(self, sock_or_address):
        Connection.__init__(self, sock_or_address)
        self.currentmessage = PyMessage()

    def handle_incoming_data(self, data):
        while data:
            done, data = self.currentmessage.feed(data)
            if done:
                import cPickle
                self.handle_object(cPickle.loads(self.currentmessage.data))
                self.currentmessage = PyMessage()

    def handle_object(self, object):
        print 'unhandled object:', repr(object)

    def send(self, object):
        import cPickle, zlib, struct
        data = cPickle.dumps(object, 1)
        length = len(data)
        checksum = zlib.adler32(data)
        data = struct.pack("ll", length, checksum) + data
        Connection.send(self, data)


class Echo(Connection):

    """Simple echoing connection: it sends everything back it receives."""

    def handle_incoming_data(self, data):
        self.send(data)


class Proxy(Connection):

    """Generic proxy connection"""

    def __init__(self, sock_or_address=None, proxyaddr=None, closepartner=0):
        """arguments:
        - sock_or_address is either a socket or a tuple containing the
        name and port number of a remote host
        - proxyaddr: a tuple containing a name and a port number of a
          remote host (optional).
        - closepartner: boolean, specifies whether we should close
          the proxy connection (optional)"""

        Connection.__init__(self, sock_or_address)
        self.other = None
        self.proxyaddr = proxyaddr
        self.closepartner = closepartner

    def close(self):
        if self.other:
            other = self.other
            self.other = None
            other.other = None
            if self.closepartner:
                other.close()
        Connection.close(self)

    def handle_incoming_data(self, data):
        if not self.other:
            # pass data for possible automatic remote host detection
            # (see HTTPProxy)
            data = self.connectproxy(data)
        self.other.send(data)

    def connectproxy(self, data):
        other = self.__class__(self.proxyaddr, closepartner=self.closepartner)
        self.other = other
        other.other = self
        return data


class HTTPProxy(Proxy):

    """Simple, useless, http proxy. It figures out itself where to connect to."""

    def connectproxy(self, data):
        if VERBOSE:
            print "--- proxy request", repr(data)
        addr, data = de_proxify(data)
        other = Proxy(addr)
        self.other = other
        other.other = self
        return data


# helper for HTTPProxy
def de_proxify(data):
    import re
    req_pattern = "GET http://([a-zA-Z0-9-_.]+)(:([0-9]+))?"
    m = re.match(req_pattern, data)
    host, dummy, port = m.groups()
    if not port:
        port = 80
    else:
        port = int(port)
    # change "GET http://xx.xx.xx/yy" into "GET /yy"
    data = re.sub(req_pattern, "GET ", data)
    return (host, port), data


# if we're running "under W", let's register the socket poller to the event loop
try:
    import W
except:
    pass
else:
    W.getapplication().addidlefunc(asyncore.poll)


## testing muck
#testserver = Server(10000, Connection)
#echoserver = Server(10007, Echo)
#httpproxyserver = Server(8088, HTTPProxy, 5)
#asyncore.close_all()



# Copyright (C) 2004 Python Software Foundation
# Author: barry@python.org (Barry Warsaw)
# License: http://www.opensource.org/licenses/PythonSoftFoundation.php

import unittest
from string import Template, SafeTemplate

class TestTemplate(unittest.TestCase):

    def test_regular_templates(self):
        s = Template('$who likes to eat a bag of $what worth $$100')
        self.assertEqual(s % dict(who='tim', what='ham'),
                         'tim likes to eat a bag of ham worth $100')
        self.assertRaises(KeyError, lambda s, d: s % d, s, dict(who='tim'))

    def test_regular_templates_with_braces(self):
        s = Template('$who likes ${what} for ${meal}')
        self.assertEqual(s % dict(who='tim', what='ham', meal='dinner'),
                         'tim likes ham for dinner')
        self.assertRaises(KeyError, lambda s, d: s % d,
                          s, dict(who='tim', what='ham'))

    def test_escapes(self):
        eq = self.assertEqual
        s = Template('$who likes to eat a bag of $$what worth $$100')
        eq(s % dict(who='tim', what='ham'),
           'tim likes to eat a bag of $what worth $100')
        s = Template('$who likes $$')
        eq(s % dict(who='tim', what='ham'), 'tim likes $')

    def test_percents(self):
        s = Template('%(foo)s $foo ${foo}')
        self.assertEqual(s % dict(foo='baz'), '%(foo)s baz baz')
        s = SafeTemplate('%(foo)s $foo ${foo}')
        self.assertEqual(s % dict(foo='baz'), '%(foo)s baz baz')

    def test_stringification(self):
        s = Template('tim has eaten $count bags of ham today')
        self.assertEqual(s % dict(count=7),
                         'tim has eaten 7 bags of ham today')
        s = SafeTemplate('tim has eaten $count bags of ham today')
        self.assertEqual(s % dict(count=7),
                         'tim has eaten 7 bags of ham today')
        s = SafeTemplate('tim has eaten ${count} bags of ham today')
        self.assertEqual(s % dict(count=7),
                         'tim has eaten 7 bags of ham today')

    def test_SafeTemplate(self):
        eq = self.assertEqual
        s = SafeTemplate('$who likes ${what} for ${meal}')
        eq(s % dict(who='tim'),
           'tim likes ${what} for ${meal}')
        eq(s % dict(what='ham'),
           '$who likes ham for ${meal}')
        eq(s % dict(what='ham', meal='dinner'),
           '$who likes ham for dinner')
        eq(s % dict(who='tim', what='ham'),
           'tim likes ham for ${meal}')
        eq(s % dict(who='tim', what='ham', meal='dinner'),
           'tim likes ham for dinner')

    def test_invalid_placeholders(self):
        raises = self.assertRaises
        s = Template('$who likes $')
        raises(ValueError, lambda s, d: s % d, s, dict(who='tim'))
        s = Template('$who likes ${what)')
        raises(ValueError, lambda s, d: s % d, s, dict(who='tim'))
        s = Template('$who likes $100')
        raises(ValueError, lambda s, d: s % d, s, dict(who='tim'))


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestTemplate))
    return suite


def test_main():
    from test import test_support
    test_support.run_suite(suite())


if __name__ == '__main__':
    unittest.main()



# This script will generate the Resources interface for Python.
# It uses the "bgen" package to generate C code.
# It execs the file resgen.py which contain the function definitions
# (resgen.py was generated by resscan.py, scanning the <Resources.h> header file).

from macsupport import *

class ResMixIn:

    def checkit(self):
        if self.returntype.__class__ != OSErrType:
            OutLbrace()
            Output("OSErr _err = ResError();")
            Output("if (_err != noErr) return PyMac_Error(_err);")
            OutRbrace()
        FunctionGenerator.checkit(self) # XXX

class ResFunction(ResMixIn, OSErrWeakLinkFunctionGenerator): pass
class ResMethod(ResMixIn, OSErrWeakLinkMethodGenerator): pass

RsrcChainLocation = Type("RsrcChainLocation", "h")
FSCatalogInfoBitmap = FakeType("0") # Type("FSCatalogInfoBitmap", "l")
FSCatalogInfo_ptr = FakeType("(FSCatalogInfo *)0")

# includestuff etc. are imported from macsupport

includestuff = includestuff + """
#include <Carbon/Carbon.h>

#ifdef USE_TOOLBOX_OBJECT_GLUE
extern PyObject *_ResObj_New(Handle);
extern int _ResObj_Convert(PyObject *, Handle *);
extern PyObject *_OptResObj_New(Handle);
extern int _OptResObj_Convert(PyObject *, Handle *);
#define ResObj_New _ResObj_New
#define ResObj_Convert _ResObj_Convert
#define OptResObj_New _OptResObj_New
#define OptResObj_Convert _OptResObj_Convert
#endif

/* Function to dispose a resource, with a "normal" calling sequence */
static void
PyMac_AutoDisposeHandle(Handle h)
{
        DisposeHandle(h);
}
"""

finalstuff = finalstuff + """

/* Alternative version of ResObj_New, which returns None for null argument */
PyObject *OptResObj_New(Handle itself)
{
        if (itself == NULL) {
                Py_INCREF(Py_None);
                return Py_None;
        }
        return ResObj_New(itself);
}

int OptResObj_Convert(PyObject *v, Handle *p_itself)
{
        PyObject *tmp;

        if ( v == Py_None ) {
                *p_itself = NULL;
                return 1;
        }
        if (ResObj_Check(v))
        {
                *p_itself = ((ResourceObject *)v)->ob_itself;
                return 1;
        }
        /* If it isn't a resource yet see whether it is convertible */
        if ( (tmp=PyObject_CallMethod(v, "as_Resource", "")) ) {
                *p_itself = ((ResourceObject *)tmp)->ob_itself;
                Py_DECREF(tmp);
                return 1;
        }
        PyErr_Clear();
        PyErr_SetString(PyExc_TypeError, "Resource required");
        return 0;
}
"""

initstuff = initstuff + """
        PyMac_INIT_TOOLBOX_OBJECT_NEW(Handle, ResObj_New);
        PyMac_INIT_TOOLBOX_OBJECT_CONVERT(Handle, ResObj_Convert);
        PyMac_INIT_TOOLBOX_OBJECT_NEW(Handle, OptResObj_New);
        PyMac_INIT_TOOLBOX_OBJECT_CONVERT(Handle, OptResObj_Convert);
"""

module = MacModule('_Res', 'Res', includestuff, finalstuff, initstuff)

class ResDefinition(PEP253Mixin, GlobalObjectDefinition):
    getsetlist = [
            ('data',
            """
            PyObject *res;
            char state;

            state = HGetState(self->ob_itself);
            HLock(self->ob_itself);
            res = PyString_FromStringAndSize(
                    *self->ob_itself,
                    GetHandleSize(self->ob_itself));
            HUnlock(self->ob_itself);
            HSetState(self->ob_itself, state);
            return res;
            """,
            """
            char *data;
            long size;

            if ( v == NULL )
                    return -1;
            if ( !PyString_Check(v) )
                    return -1;
            size = PyString_Size(v);
            data = PyString_AsString(v);
            /* XXXX Do I need the GetState/SetState calls? */
            SetHandleSize(self->ob_itself, size);
            if ( MemError())
                    return -1;
            HLock(self->ob_itself);
            memcpy((char *)*self->ob_itself, data, size);
            HUnlock(self->ob_itself);
            /* XXXX Should I do the Changed call immedeately? */
            return 0;
            """,
            'The resource data'
            ), (
            'size',
            'return PyInt_FromLong(GetHandleSize(self->ob_itself));',
            None,
            'The length of the resource data'
            )]

    def outputCheckNewArg(self):
        Output("if (itself == NULL) return PyMac_Error(resNotFound);")

    def outputCheckConvertArg(self):
        # if it isn't a resource we may be able to coerce it
        Output("if (!%s_Check(v))", self.prefix)
        OutLbrace()
        Output("PyObject *tmp;")
        Output('if ( (tmp=PyObject_CallMethod(v, "as_Resource", "")) )')
        OutLbrace()
        Output("*p_itself = ((ResourceObject *)tmp)->ob_itself;")
        Output("Py_DECREF(tmp);")
        Output("return 1;")
        OutRbrace()
        Output("PyErr_Clear();")
        OutRbrace()

    def outputStructMembers(self):
        GlobalObjectDefinition.outputStructMembers(self)
        Output("void (*ob_freeit)(%s ptr);", self.itselftype)

    def outputInitStructMembers(self):
        GlobalObjectDefinition.outputInitStructMembers(self)
        Output("it->ob_freeit = NULL;")

    def outputCleanupStructMembers(self):
        Output("if (self->ob_freeit && self->ob_itself)")
        OutLbrace()
        Output("self->ob_freeit(self->ob_itself);")
        OutRbrace()
        Output("self->ob_itself = NULL;")

    def output_tp_newBody(self):
        Output("PyObject *self;")
        Output
        Output("if ((self = type->tp_alloc(type, 0)) == NULL) return NULL;")
        Output("((%s *)self)->ob_itself = NULL;", self.objecttype)
        Output("((%s *)self)->ob_freeit = NULL;", self.objecttype)
        Output("return self;")

    def output_tp_initBody(self):
        Output("char *srcdata = NULL;")
        Output("int srclen = 0;")
        Output("%s itself;", self.itselftype);
        Output("char *kw[] = {\"itself\", 0};")
        Output()
        Output("if (PyArg_ParseTupleAndKeywords(args, kwds, \"O&\", kw, %s_Convert, &itself))",
                self.prefix);
        OutLbrace()
        Output("((%s *)self)->ob_itself = itself;", self.objecttype)
        Output("return 0;")
        OutRbrace()
        Output("PyErr_Clear();")
        Output("if (!PyArg_ParseTupleAndKeywords(args, kwds, \"|s#\", kw, &srcdata, &srclen)) return -1;")
        Output("if ((itself = NewHandle(srclen)) == NULL)")
        OutLbrace()
        Output("PyErr_NoMemory();")
        Output("return 0;")
        OutRbrace()
        Output("((%s *)self)->ob_itself = itself;", self.objecttype)
# XXXX          Output("((%s *)self)->ob_freeit = PyMac_AutoDisposeHandle;")
        Output("if (srclen && srcdata)")
        OutLbrace()
        Output("HLock(itself);")
        Output("memcpy(*itself, srcdata, srclen);")
        Output("HUnlock(itself);")
        OutRbrace()
        Output("return 0;")

resobject = ResDefinition('Resource', 'ResObj', 'Handle')
module.addobject(resobject)

functions = []
resmethods = []

execfile('resgen.py')
execfile('resedit.py')

for f in functions: module.add(f)
for f in resmethods: resobject.add(f)

SetOutputFileName('_Resmodule.c')
module.generate()



from Carbon import Dlg
from Carbon import Res

splash = Dlg.GetNewDialog(468, -1)
splash.DrawDialog()

from Carbon import Qd, TE, Fm

from Carbon import Win
from Carbon.Fonts import *
from Carbon.QuickDraw import *
from Carbon.TextEdit import teJustCenter
import string
import sys

_about_width = 440
_about_height = 340

_keepsplashscreenopen = 0

abouttext1 = """The Python Integrated Development Environment for the Macintosh\xaa
Version: %s
Copyright 1997-2001 Just van Rossum, Letterror. <just@letterror.com>
Python %s
%s
See: <http://www.python.org/> for information and documentation."""

flauwekul = [   "Goodday, Bruce.",
                        "What's new?",
                        "Nudge, nudge, say no more!",
                        "No, no sir, it's not dead. It's resting.",
                        "Albatros!",
                        "It's . . .",
                        "Is your name not Bruce, then?",
                        """But Mr F.G. Superman has a secret identity . . .
when trouble strikes at any time . . .
at any place . . . he is ready to become . . .
Bicycle Repair Man!"""
                        ]

def skipdoublereturns(text):
    return string.replace(text, '\n\n', '\n')

def nl2return(text):
    return string.replace(text, '\n', '\r')

def UpdateSplash(drawdialog = 0, what = 0):
    if drawdialog:
        splash.DrawDialog()
    drawtext(what)
    splash.GetDialogWindow().ValidWindowRect(splash.GetDialogPort().GetPortBounds())
    splash.GetDialogWindow().GetWindowPort().QDFlushPortBuffer(None)

def drawtext(what = 0):
    Qd.SetPort(splash)
    fontID = Fm.GetFNum("Python-Sans")
    if not fontID:
        fontID = geneva
    Qd.TextFont(fontID)
    Qd.TextSize(9)
    rect = (10, 115, _about_width - 10, _about_height - 30)
    if not what:
        import __main__
        abouttxt = nl2return(abouttext1 % (
                        __main__.__version__, sys.version, skipdoublereturns(sys.copyright)))
    else:
        import random
        abouttxt = nl2return(random.choice(flauwekul))
    TE.TETextBox(abouttxt, rect, teJustCenter)

UpdateSplash(1)

def wait():
    from Carbon import Evt
    from Carbon import Events
    global splash
    try:
        splash
    except NameError:
        return
    Qd.InitCursor()
    time = Evt.TickCount()
    whattext = 0
    drawtext(whattext)
    while _keepsplashscreenopen:
        ok, event = Evt.EventAvail(Events.highLevelEventMask)
        if ok:
            # got apple event, back to mainloop
            break
        ok, event = Evt.EventAvail(Events.mDownMask | Events.keyDownMask | Events.updateMask)
        if ok:
            ok, event = Evt.WaitNextEvent(Events.mDownMask | Events.keyDownMask | Events.updateMask, 30)
            if ok:
                (what, message, when, where, modifiers) = event
                if what == Events.updateEvt:
                    if Win.WhichWindow(message) == splash:
                        UpdateSplash(1, whattext)
                else:
                    break
        if Evt.TickCount() - time > 360:
            whattext = not whattext
            drawtext(whattext)
            time = Evt.TickCount()
    del splash


def about():
    global splash, splashresfile, _keepsplashscreenopen
    _keepsplashscreenopen = 1
    splash = Dlg.GetNewDialog(468, -1)
    splash.DrawDialog()
    wait()



"""Minimal W application."""

import Wapplication
import macresource
import os

class TestApp(Wapplication.Application):

    def __init__(self):
        from Carbon import Res
#               macresource.open_pathname("Widgets.rsrc")
        self._menustocheck = []
        self.preffilepath = os.path.join("Python", "PythonIDE preferences")
        Wapplication.Application.__init__(self, 'Pyth')
        # open a new text editor
        import PyEdit
        PyEdit.Editor()
        # start the mainloop
        self.mainloop()


TestApp()



"""macmodulefinder - Find modules used in a script. Only slightly
mac-specific, really."""

import sys
import os

import directives

try:
    # This will work if we are frozen ourselves
    import modulefinder
except ImportError:
    # And this will work otherwise
    _FREEZEDIR=os.path.join(sys.prefix, ':Tools:freeze')
    sys.path.insert(0, _FREEZEDIR)
    import modulefinder

#
# Modules that must be included, and modules that need not be included
# (but are if they are found)
#
MAC_INCLUDE_MODULES=['site']
MAC_MAYMISS_MODULES=['posix', 'os2', 'nt', 'ntpath', 'dos', 'dospath',
                'win32api', 'ce', '_winreg',
                'nturl2path', 'pwd', 'sitecustomize',
                'org.python.core',
                'riscos', 'riscosenviron', 'riscospath'
                ]

# An exception:
Missing="macmodulefinder.Missing"

class Module(modulefinder.Module):

    def gettype(self):
        """Return type of module"""
        if self.__path__:
            return 'package'
        if self.__code__:
            return 'module'
        if self.__file__:
            return 'dynamic'
        return 'builtin'

class ModuleFinder(modulefinder.ModuleFinder):

    def add_module(self, fqname):
        if self.modules.has_key(fqname):
            return self.modules[fqname]
        self.modules[fqname] = m = Module(fqname)
        return m

def process(program, modules=None, module_files=None, debug=0):
    if modules is None:
        modules = []
    if module_files is None:
        module_files = []
    missing = []
    #
    # Add the standard modules needed for startup
    #
    modules = modules + MAC_INCLUDE_MODULES
    #
    # search the main source for directives
    #
    extra_modules, exclude_modules, optional_modules, extra_path = \
                    directives.findfreezedirectives(program)
    for m in extra_modules:
        if os.sep in m:
            # It is a file
            module_files.append(m)
        else:
            modules.append(m)
    # collect all modules of the program
    path = sys.path[:]
    dir = os.path.dirname(program)
    path[0] = dir   # "current dir"
    path = extra_path + path
    #
    # Create the module finder and let it do its work
    #
    modfinder = ModuleFinder(path,
                    excludes=exclude_modules, debug=debug)
    for m in modules:
        modfinder.import_hook(m)
    for m in module_files:
        modfinder.load_file(m)
    modfinder.run_script(program)
    module_dict = modfinder.modules
    #
    # Tell the user about missing modules
    #
    maymiss = exclude_modules + optional_modules + MAC_MAYMISS_MODULES
    for m in modfinder.badmodules.keys():
        if not m in maymiss:
            if debug > 0:
                print 'Missing', m
            missing.append(m)
    #
    # Warn the user about unused builtins
    #
    for m in sys.builtin_module_names:
        if m in ('__main__', '__builtin__'):
            pass
        elif not module_dict.has_key(m):
            if debug > 0:
                print 'Unused', m
        elif module_dict[m].gettype() != 'builtin':
            # XXXX Can this happen?
            if debug > 0:
                print 'Conflict', m
    return module_dict, missing



# This is a temporary setup script to allow distribution of
# MacPython 2.4 modules for MacPython 2.3.

from distutils.core import Extension, setup

setup(name="LaunchServices", version="0.2",
        ext_modules=[
                Extension('_Launch', ['_Launchmodule.c'],
                extra_link_args=['-framework', 'ApplicationServices'])
        ],
        py_modules=['LaunchServices.Launch', 'LaunchServices.LaunchServices'],
        package_dir={'LaunchServices':'../../../Lib/plat-mac/Carbon'}
        )



#!/usr/bin/env python
#
# test_codecmaps_cn.py
#   Codec mapping tests for PRC encodings
#
# $CJKCodecs: test_codecmaps_cn.py,v 1.3 2004/06/19 06:09:55 perky Exp $

from test import test_support
from test import test_multibytecodec_support
import unittest

class TestGB2312Map(test_multibytecodec_support.TestBase_Mapping,
                   unittest.TestCase):
    encoding = 'gb2312'
    mapfilename = 'EUC-CN.TXT'
    mapfileurl = 'http://people.freebsd.org/~perky/i18n/EUC-CN.TXT'

class TestGBKMap(test_multibytecodec_support.TestBase_Mapping,
                   unittest.TestCase):
    encoding = 'gbk'
    mapfilename = 'CP936.TXT'
    mapfileurl = 'http://www.unicode.org/Public/MAPPINGS/VENDORS/' \
                 'MICSFT/WINDOWS/CP936.TXT'

def test_main():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestGB2312Map))
    suite.addTest(unittest.makeSuite(TestGBKMap))
    test_support.run_suite(suite)

test_multibytecodec_support.register_skip_expected(TestGB2312Map, TestGBKMap)
if __name__ == "__main__":
    test_main()



"""imgbrowse - Display pictures using img"""

import FrameWork
import EasyDialogs
from Carbon import Res
from Carbon import Qd
from Carbon import QuickDraw
from Carbon import Win
#ifrom Carbon mport List
import sys
import struct
import img
import imgformat
import struct
import mac_image


# Where is the picture window?
LEFT=200
TOP=64
MINWIDTH=64
MINHEIGHT=64
MAXWIDTH=320
MAXHEIGHT=320


def main():
    print 'hello world'
    imgbrowse()

class imgbrowse(FrameWork.Application):
    def __init__(self):
        # First init menus, etc.
        FrameWork.Application.__init__(self)
        self.lastwin = None
        # Finally, go into the event loop
        self.mainloop()

    def makeusermenus(self):
        self.filemenu = m = FrameWork.Menu(self.menubar, "File")
        self.openitem = FrameWork.MenuItem(m, "Open...", "O", self.opendoc)
        self.infoitem = FrameWork.MenuItem(m, "Info", "I", self.info)
        self.quititem = FrameWork.MenuItem(m, "Quit", "Q", self.quit)

    def quit(self, *args):
        self._quit()

    def opendoc(self, *args):
        pathname = EasyDialogs.AskFileForOpen() # Any file type
        if not pathname:
            return
        bar = EasyDialogs.ProgressBar('Reading and converting...')
        try:
            rdr = img.reader(imgformat.macrgb16, pathname)
        except img.error, arg:
            EasyDialogs.Message(repr(arg))
            return
        w, h = rdr.width, rdr.height
        bar.set(10)
        data = rdr.read()
        del bar
        pixmap = mac_image.mkpixmap(w, h, imgformat.macrgb16, data)
        self.showimg(w, h, pixmap, data)

    def showimg(self, w, h, pixmap, data):
        win = imgwindow(self)
        win.open(w, h, pixmap, data)
        self.lastwin = win

    def info(self, *args):
        if self.lastwin:
            self.lastwin.info()

class imgwindow(FrameWork.Window):
    def open(self, width, height, pixmap, data):
        self.pixmap = pixmap
        self.data = data
        self.pictrect = (0, 0, width, height)
        bounds = (LEFT, TOP, LEFT+width, TOP+height)

        self.wid = Win.NewCWindow(bounds, "Picture", 1, 0, -1, 1, 0)
        self.do_postopen()

    def do_update(self, *args):
        pass
        currect = self.fitrect()
        print 'PICT:', self.pictrect
        print 'WIND:', currect
        print 'ARGS:', (self.pixmap, self.wid.GetWindowPort().GetPortBitMapForCopyBits(), self.pictrect,
                        currect, QuickDraw.srcCopy, None)
        self.info()
        Qd.CopyBits(self.pixmap, self.wid.GetWindowPort().GetPortBitMapForCopyBits(), self.pictrect,
                        currect, QuickDraw.srcCopy, None)

    def fitrect(self):
        """Return self.pictrect scaled to fit in window"""
        graf = self.wid.GetWindowPort()
        screenrect = graf.GetPortBounds()
        picwidth = self.pictrect[2] - self.pictrect[0]
        picheight = self.pictrect[3] - self.pictrect[1]
        if picwidth > screenrect[2] - screenrect[0]:
            factor = float(picwidth) / float(screenrect[2]-screenrect[0])
            picwidth = picwidth / factor
            picheight = picheight / factor
        if picheight > screenrect[3] - screenrect[1]:
            factor = float(picheight) / float(screenrect[3]-screenrect[1])
            picwidth = picwidth / factor
            picheight = picheight / factor
        return (screenrect[0], screenrect[1], screenrect[0]+int(picwidth),
                        screenrect[1]+int(picheight))

    def info(self):
        graf = self.wid.GetWindowPort()
        bits = graf.GetPortBitMapForCopyBits()
        mac_image.dumppixmap(bits.pixmap_data)

main()



import unittest
from test import test_support, list_tests

class ListTest(list_tests.CommonTest):
    type2test = list

    def test_truth(self):
        super(ListTest, self).test_truth()
        self.assert_(not [])
        self.assert_([42])

    def test_identity(self):
        self.assert_([] is not [])

    def test_len(self):
        super(ListTest, self).test_len()
        self.assertEqual(len([]), 0)
        self.assertEqual(len([0]), 1)
        self.assertEqual(len([0, 1, 2]), 3)

def test_main():
    test_support.run_unittest(ListTest)

if __name__=="__main__":
    test_main()



spacekey                = ' '
returnkey               = '\r'
tabkey          = '\t'
enterkey                = '\003'
backspacekey    = '\010'
deletekey               = '\177'
clearkey                = '\033'
helpkey                 = '\005'

leftarrowkey    = '\034'
rightarrowkey   = '\035'
uparrowkey              = '\036'
downarrowkey    = '\037'
arrowkeys               = [leftarrowkey, rightarrowkey, uparrowkey, downarrowkey]

topkey          = '\001'
bottomkey               = '\004'
pageupkey               = '\013'
pagedownkey     = '\014'
scrollkeys              = [topkey, bottomkey, pageupkey, pagedownkey]

navigationkeys = arrowkeys + scrollkeys

keycodes = {
        "space"         : ' ',
        "return"                : '\r',
        "tab"                   : '\t',
        "enter"                 : '\003',
        "backspace"     : '\010',
        "delete"                : '\177',
        "help"          : '\005',
        "leftarrow"             : '\034',
        "rightarrow"    : '\035',
        "uparrow"               : '\036',
        "downarrow"     : '\037',
        "top"                   : '\001',
        "bottom"                : '\004',
        "pageup"                : '\013',
        "pagedown"      : '\014'
}

keynames = {}
for k, v in keycodes.items():
    keynames[v] = k
del k, v



import W
from Carbon import Evt
import EasyDialogs

import sys
import StringIO
import string
import pstats, fpformat

# increase precision
def f8(x):
    return string.rjust(fpformat.fix(x, 4), 8)
pstats.f8 = f8

# hacking around a hack
if sys.version[:3] > '1.4':
    timer = Evt.TickCount
else:
    def timer(TickCount = Evt.TickCount):
        return TickCount() / 60.0

class ProfileBrowser:

    def __init__(self, stats = None):
        self.sortkeys = ('calls',)
        self.setupwidgets()
        self.setstats(stats)

    def setupwidgets(self):
        self.w = W.Window((580, 400), "Profile Statistics", minsize = (200, 100), tabbable = 0)
        self.w.divline = W.HorizontalLine((0, 20, 0, 0))
        self.w.titlebar = W.TextBox((4, 4, 40, 12), 'Sort by:')
        self.buttons = []
        x = 54
        width1 = 50
        width2 = 75
        for name in ["calls", "time", "cumulative", "stdname", "file", "line", "name"]:
            if len(name) > 6:
                width = width2
            else:
                width = width1
            self.w["button_" + name] = W.RadioButton((x, 4, width, 12), name, self.buttons, self.setsort)
            x += width + 10
        self.w.button_calls.set(1)
        self.w.text = W.TextEditor((0, 21, -15, -15), inset = (6, 5),
                        readonly = 1, wrap = 0, fontsettings = ('Monaco', 0, 9, (0, 0, 0)))
        self.w._bary = W.Scrollbar((-15, 20, 16, -14), self.w.text.vscroll, max = 32767)
        self.w._barx = W.Scrollbar((-1, -15, -14, 16), self.w.text.hscroll, max = 32767)
        self.w.open()

    def setstats(self, stats):
        self.stats = stats
        self.stats.strip_dirs()
        self.displaystats()

    def setsort(self):
        # Grmpf. The callback doesn't give us the button:-(
        for b in self.buttons:
            if b.get():
                if b._title == self.sortkeys[0]:
                    return
                self.sortkeys = (b._title,) + self.sortkeys[:3]
                break
        self.displaystats()

    def displaystats(self):
        W.SetCursor('watch')
        apply(self.stats.sort_stats, self.sortkeys)
        saveout = sys.stdout
        try:
            s = sys.stdout = StringIO.StringIO()
            self.stats.print_stats()
        finally:
            sys.stdout = saveout
        text = string.join(string.split(s.getvalue(), '\n'), '\r')
        self.w.text.set(text)


def main():
    import pstats
    args = sys.argv[1:]
    for i in args:
        stats = pstats.Stats(i)
        browser = ProfileBrowser(stats)
    else:
        filename = EasyDialogs.AskFileForOpen(message='Profiler data')
        if not filename: sys.exit(0)
        stats = pstats.Stats(filename)
        browser = ProfileBrowser(stats)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
#
# test_multibytecodec_support.py
#   Common Unittest Routines for CJK codecs
#
# $CJKCodecs: test_multibytecodec_support.py,v 1.6 2004/06/19 06:09:55 perky Exp $

import sys, codecs, os.path
import unittest
from test import test_support
from StringIO import StringIO

__cjkcodecs__ = 0 # define this as 0 for python

class TestBase:
    encoding        = ''   # codec name
    codec           = None # codec tuple (with 4 elements)
    tstring         = ''   # string to test StreamReader

    codectests      = None # must set. codec test tuple
    roundtriptest   = 1    # set if roundtrip is possible with unicode
    has_iso10646    = 0    # set if this encoding contains whole iso10646 map
    xmlcharnametest = None # string to test xmlcharrefreplace

    def setUp(self):
        if self.codec is None:
            self.codec = codecs.lookup(self.encoding)
        self.encode, self.decode, self.reader, self.writer = self.codec

    def test_chunkcoding(self):
        for native, utf8 in zip(*[StringIO(f).readlines()
                                  for f in self.tstring]):
            u = self.decode(native)[0]
            self.assertEqual(u, utf8.decode('utf-8'))
            if self.roundtriptest:
                self.assertEqual(native, self.encode(u)[0])

    def test_errorhandle(self):
        for source, scheme, expected in self.codectests:
            if type(source) == type(''):
                func = self.decode
            else:
                func = self.encode
            if expected:
                result = func(source, scheme)[0]
                self.assertEqual(result, expected)
            else:
                self.assertRaises(UnicodeError, func, source, scheme)

    if sys.hexversion >= 0x02030000:
        def test_xmlcharrefreplace(self):
            if self.has_iso10646:
                return

            s = u"\u0b13\u0b23\u0b60 nd eggs"
            self.assertEqual(
                self.encode(s, "xmlcharrefreplace")[0],
                "&#2835;&#2851;&#2912; nd eggs"
            )

        def test_customreplace(self):
            if self.has_iso10646:
                return

            import htmlentitydefs

            names = {}
            for (key, value) in htmlentitydefs.entitydefs.items():
                if len(value)==1:
                    names[value.decode('latin-1')] = self.decode(key)[0]
                else:
                    names[unichr(int(value[2:-1]))] = self.decode(key)[0]

            def xmlcharnamereplace(exc):
                if not isinstance(exc, UnicodeEncodeError):
                    raise TypeError("don't know how to handle %r" % exc)
                l = []
                for c in exc.object[exc.start:exc.end]:
                    try:
                        l.append(u"&%s;" % names[c])
                    except KeyError:
                        l.append(u"&#%d;" % ord(c))
                return (u"".join(l), exc.end)

            codecs.register_error(
                "test.xmlcharnamereplace", xmlcharnamereplace)

            if self.xmlcharnametest:
                sin, sout = self.xmlcharnametest
            else:
                sin = u"\xab\u211c\xbb = \u2329\u1234\u232a"
                sout = "&laquo;&real;&raquo; = &lang;&#4660;&rang;"
            self.assertEqual(self.encode(sin,
                                        "test.xmlcharnamereplace")[0], sout)

    def test_streamreader(self):
        UTF8Writer = codecs.getwriter('utf-8')
        for name in ["read", "readline", "readlines"]:
            for sizehint in [None, -1] + range(1, 33) + \
                            [64, 128, 256, 512, 1024]:
                istream = self.reader(StringIO(self.tstring[0]))
                ostream = UTF8Writer(StringIO())
                func = getattr(istream, name)
                while 1:
                    data = func(sizehint)
                    if not data:
                        break
                    if name == "readlines":
                        ostream.writelines(data)
                    else:
                        ostream.write(data)

                self.assertEqual(ostream.getvalue(), self.tstring[1])

    def test_streamwriter(self):
        if __cjkcodecs__:
            readfuncs = ('read', 'readline', 'readlines')
        else:
            # standard utf8 codec has broken readline and readlines.
            readfuncs = ('read',)
        UTF8Reader = codecs.getreader('utf-8')
        for name in readfuncs:
            for sizehint in [None] + range(1, 33) + \
                            [64, 128, 256, 512, 1024]:
                istream = UTF8Reader(StringIO(self.tstring[1]))
                ostream = self.writer(StringIO())
                func = getattr(istream, name)
                while 1:
                    if sizehint is not None:
                        data = func(sizehint)
                    else:
                        data = func()

                    if not data:
                        break
                    if name == "readlines":
                        ostream.writelines(data)
                    else:
                        ostream.write(data)

                self.assertEqual(ostream.getvalue(), self.tstring[0])

if len(u'\U00012345') == 2: # ucs2 build
    _unichr = unichr
    def unichr(v):
        if v >= 0x10000:
            return _unichr(0xd800 + ((v - 0x10000) >> 10)) + \
                   _unichr(0xdc00 + ((v - 0x10000) & 0x3ff))
        else:
            return _unichr(v)
    _ord = ord
    def ord(c):
        if len(c) == 2:
            return 0x10000 + ((_ord(c[0]) - 0xd800) << 10) + \
                          (ord(c[1]) - 0xdc00)
        else:
            return _ord(c)

class TestBase_Mapping(unittest.TestCase):
    pass_enctest = []
    pass_dectest = []
    supmaps = []

    def __init__(self, *args, **kw):
        unittest.TestCase.__init__(self, *args, **kw)
        if not os.path.exists(self.mapfilename):
            raise test_support.TestSkipped('%s not found, download from %s' %
                    (self.mapfilename, self.mapfileurl))

    def test_mapping_file(self):
        unichrs = lambda s: u''.join(map(unichr, map(eval, s.split('+'))))
        urt_wa = {}

        for line in open(self.mapfilename):
            if not line:
                break
            data = line.split('#')[0].strip().split()
            if len(data) != 2:
                continue

            csetval = eval(data[0])
            if csetval <= 0x7F:
                csetch = chr(csetval & 0xff)
            elif csetval >= 0x1000000:
                csetch = chr(csetval >> 24) + chr((csetval >> 16) & 0xff) + \
                         chr((csetval >> 8) & 0xff) + chr(csetval & 0xff)
            elif csetval >= 0x10000:
                csetch = chr(csetval >> 16) + \
                         chr((csetval >> 8) & 0xff) + chr(csetval & 0xff)
            elif csetval >= 0x100:
                csetch = chr(csetval >> 8) + chr(csetval & 0xff)
            else:
                continue

            unich = unichrs(data[1])
            if ord(unich) == 0xfffd or urt_wa.has_key(unich):
                continue
            urt_wa[unich] = csetch

            self._testpoint(csetch, unich)

    def test_mapping_supplemental(self):
        for mapping in self.supmaps:
            self._testpoint(*mapping)

    def _testpoint(self, csetch, unich):
        if (csetch, unich) not in self.pass_enctest:
            self.assertEqual(unich.encode(self.encoding), csetch)
        if (csetch, unich) not in self.pass_dectest:
            self.assertEqual(unicode(csetch, self.encoding), unich)

def load_teststring(encoding):
    if __cjkcodecs__:
        etxt = open(os.path.join('sampletexts', encoding) + '.txt').read()
        utxt = open(os.path.join('sampletexts', encoding) + '.utf8').read()
        return (etxt, utxt)
    else:
        from test import cjkencodings_test
        return cjkencodings_test.teststring[encoding]

def register_skip_expected(*cases):
    for case in cases: # len(cases) must be 1 at least.
        for path in [os.path.curdir, os.path.pardir]:
            fn = os.path.join(path, case.mapfilename)
            if os.path.exists(fn):
                case.mapfilename = fn
                break
        else:
            sys.modules[case.__module__].skip_expected = True
            break
    else:
        sys.modules[case.__module__].skip_expected = False



# Copyright (C) 2001-2004 Python Software Foundation
# Author: barry@python.org (Barry Warsaw)

"""email package exception classes."""



class MessageError(Exception):
    """Base class for errors in the email package."""


class MessageParseError(MessageError):
    """Base class for message parsing errors."""


class HeaderParseError(MessageParseError):
    """Error while parsing headers."""


class BoundaryError(MessageParseError):
    """Couldn't find terminating boundary."""


class MultipartConversionError(MessageError, TypeError):
    """Conversion to a multipart is prohibited."""



# These are parsing defects which the parser was able to work around.
class MessageDefect:
    """Base class for a message defect."""

    def __init__(self, line=None):
        self.line = line

class NoBoundaryInMultipart(MessageDefect):
    """A message claimed to be a multipart but had no boundary parameter."""

class StartBoundaryNotFound(MessageDefect):
    """The claimed start boundary was never found."""

class FirstHeaderLineIsContinuation(MessageDefect):
    """A message had a continuation line as its first header line."""

class MisplacedEnvelopeHeader(MessageDefect):
    """A 'Unix-from' header was found in the middle of a header block."""

class MalformedHeader(MessageDefect):
    """Found a header that was missing a colon, or was otherwise malformed"""



import FrameWork
from Carbon import Win
from Carbon import Qd
from Carbon import Evt
import MacOS
from Carbon import Events
import traceback
from types import *
from Carbon import Menu; MenuToolbox = Menu; del Menu
import macresource
from Carbon import File

if hasattr(Win, "FrontNonFloatingWindow"):
    MyFrontWindow = Win.FrontNonFloatingWindow
else:
    MyFrontWindow = Win.FrontWindow


KILLUNKNOWNWINDOWS = 0  # Set to 0 for debugging.

class Application(FrameWork.Application):

    def __init__(self, signature='Pyth'):
        # Open our resource file, if it is not open yet
        macresource.need('CURS', 468, "Widgets.rsrc")
        import W
        W.setapplication(self, signature)
        FrameWork.Application.__init__(self)
        self._suspended = 0
        self.quitting = 0
        self.debugger_quitting = 1
        self.DebuggerQuit = 'DebuggerQuitDummyException'
        self._idlefuncs = []
        # map certain F key codes to equivalent command-letter combos (JJS)
        self.fkeymaps = {122:"z", 120:"x", 99:"c", 118:"v"}

    def mainloop(self, mask=FrameWork.everyEvent, wait=None):
        import W
        self.quitting = 0
        if hasattr(MacOS, 'EnableAppswitch'):
            saveyield = MacOS.EnableAppswitch(-1)
        try:
            while not self.quitting:
                try:
                    self.do1event(mask, wait)
                except W.AlertError, detail:
                    if hasattr(MacOS, 'EnableAppswitch'):
                        MacOS.EnableAppswitch(-1)
                    W.Message(detail)
                except self.DebuggerQuit:
                    if hasattr(MacOS, 'EnableAppswitch'):
                        MacOS.EnableAppswitch(-1)
                except:
                    if hasattr(MacOS, 'EnableAppswitch'):
                        MacOS.EnableAppswitch(-1)
                    import PyEdit
                    PyEdit.tracebackwindow.traceback()
        finally:
            if hasattr(MacOS, 'EnableAppswitch'):
                MacOS.EnableAppswitch(1)

    def debugger_mainloop(self, mask=FrameWork.everyEvent, wait=None):
        import W
        self.debugger_quitting = 0
        if hasattr(MacOS, 'EnableAppswitch'):
            saveyield = MacOS.EnableAppswitch(-1)
        try:
            while not self.quitting and not self.debugger_quitting:
                try:
                    self.do1event(mask, wait)
                except W.AlertError, detail:
                    W.Message(detail)
                except:
                    import PyEdit
                    PyEdit.tracebackwindow.traceback()
        finally:
            if hasattr(MacOS, 'EnableAppswitch'):
                MacOS.EnableAppswitch(saveyield)

    def breathe(self, wait=1):
        import W
        ok, event = Evt.WaitNextEvent(FrameWork.updateMask |
                        FrameWork.mDownMask | FrameWork.osMask |
                        FrameWork.activMask,
                        wait)
        if ok:
            (what, message, when, where, modifiers) = event
            #print FrameWork.eventname[what]
            if FrameWork.eventname[what] == 'mouseDown':
                partcode, wid = Win.FindWindow(where)
                if FrameWork.partname[partcode] <> 'inDesk':
                    return
            else:
                W.SetCursor('watch')
            self.dispatch(event)

    def refreshwindows(self, wait=1):
        import W
        while 1:
            ok, event = Evt.WaitNextEvent(FrameWork.updateMask, wait)
            if not ok:
                break
            self.dispatch(event)

    def addidlefunc(self, func):
        self._idlefuncs.append(func)

    def removeidlefunc(self, func):
        self._idlefuncs.remove(func)

    def idle(self, event):
        if not self._suspended:
            if not self.do_frontWindowMethod("idle", event):
                Qd.InitCursor()
        if self._idlefuncs:
            for func in self._idlefuncs:
                try:
                    func()
                except:
                    import sys
                    sys.stderr.write("exception in idle function %r; killed:\n" % (func,))
                    traceback.print_exc()
                    self._idlefuncs.remove(func)
                    break

    def do_frontWindowMethod(self, attr, *args):
        wid = MyFrontWindow()
        if wid and self._windows.has_key(wid):
            window = self._windows[wid]
            if hasattr(window, attr):
                handler = getattr(window, attr)
                apply(handler, args)
                return 1

    def getfrontwindow(self):
        wid = MyFrontWindow()
        if wid and self._windows.has_key(wid):
            return self._windows[wid]
        return None

    def appendwindow(self, wid, window):
        self._windows[wid] = window
        self.makeopenwindowsmenu()

    def removewindow(self, wid):
        del self._windows[wid]
        self.makeopenwindowsmenu()

    def makeopenwindowsmenu(self):
        # dummy; could be the full version from PythonIDEMain.py
        self._openwindows = {}
        self._openwindowscheckmark = 0
        if not hasattr(self, "_menustocheck"):
            self._menustocheck = []

    def do_key(self, event):
        (what, message, when, where, modifiers) = event
        ch = chr(message & FrameWork.charCodeMask)
        rest = message & ~FrameWork.charCodeMask
        keycode = (message & FrameWork.keyCodeMask) >> 8
        if keycode in self.fkeymaps.keys():             # JJS
            ch = self.fkeymaps[keycode]
            modifiers = modifiers | FrameWork.cmdKey
        wid = MyFrontWindow()
        if modifiers & FrameWork.cmdKey and not modifiers & FrameWork.shiftKey:
            if wid and self._windows.has_key(wid):
                self.checkmenus(self._windows[wid])
            else:
                self.checkmenus(None)
            event = (what, ord(ch) | rest, when, where, modifiers)
            result = MenuToolbox.MenuKey(ord(ch))
            id = (result>>16) & 0xffff      # Hi word
            item = result & 0xffff          # Lo word
            if id:
                self.do_rawmenu(id, item, None, event)
                return  # here! we had a menukey!
            #else:
            #       print "XXX Command-%r" % ch
        # See whether the front window wants it
        if wid and self._windows.has_key(wid):
            window = self._windows[wid]
            try:
                do_char = window.do_char
            except AttributeError:
                do_char = self.do_char
            do_char(ch, event)
        # else it wasn't for us, sigh...

    def do_inMenuBar(self, partcode, window, event):
        Qd.InitCursor()
        (what, message, when, where, modifiers) = event
        self.checkopenwindowsmenu()
        wid = MyFrontWindow()
        if wid and self._windows.has_key(wid):
            self.checkmenus(self._windows[wid])
        else:
            self.checkmenus(None)
        result = MenuToolbox.MenuSelect(where)
        id = (result>>16) & 0xffff      # Hi word
        if id >= 0x8000:
            id = -0x10000 + id
        item = result & 0xffff          # Lo word
        self.do_rawmenu(id, item, window, event)

    def do_updateEvt(self, event):
        (what, message, when, where, modifiers) = event
        wid = Win.WhichWindow(message)
        if wid and self._windows.has_key(wid):
            window = self._windows[wid]
            window.do_rawupdate(wid, event)
        else:
            if KILLUNKNOWNWINDOWS and wid:
                wid.HideWindow()
                import sys
                sys.stderr.write("XXX killed unknown (crashed?) Python window.\n")
            else:
                if hasattr(MacOS, 'HandleEvent'):
                    MacOS.HandleEvent(event)
                else:
                    print 'Unexpected updateEvent:', event

    def suspendresume(self, onoff):
        pass

    def do_suspendresume(self, event):
        self._suspended = not event[1] & 1
        FrameWork.Application.do_suspendresume(self, event)

    def checkopenwindowsmenu(self):
        if self._openwindowscheckmark:
            self.openwindowsmenu.menu.CheckMenuItem(self._openwindowscheckmark, 0)
        window = MyFrontWindow()
        if window:
            for item, wid in self._openwindows.items():
                if wid == window:
                    #self.pythonwindowsmenuitem.check(1)
                    self.openwindowsmenu.menu.CheckMenuItem(item, 1)
                    self._openwindowscheckmark = item
                    break
        else:
            self._openwindowscheckmark = 0
        #if self._openwindows:
        #       self.pythonwindowsmenuitem.enable(1)
        #else:
        #       self.pythonwindowsmenuitem.enable(0)

    def checkmenus(self, window):
        for item in self._menustocheck:
            callback = item.menu.items[item.item-1][2]
            if type(callback) <> StringType:
                item.enable(1)
            elif hasattr(window, "domenu_" + callback):
                if hasattr(window, "can_" + callback):
                    canhandler = getattr(window, "can_" + callback)
                    if canhandler(item):
                        item.enable(1)
                    else:
                        item.enable(0)
                else:
                    item.enable(1)
            else:
                item.enable(0)

    def enablemenubar(self, onoff):
        for m in self.menubar.menus.values():
            if onoff:
                m.menu.EnableMenuItem(0)
            elif m.menu.GetMenuItemText(3) <> 'Cut': # ew...
                m.menu.DisableMenuItem(0)
        MenuToolbox.DrawMenuBar()

    def makemenubar(self):
        self.menubar = MenuBar(self)
        FrameWork.AppleMenu(self.menubar, self.getabouttext(), self.do_about)
        self.makeusermenus()

    def scriptswalk(self, top, menu, done=None):
        if menu.id > 200:
            import W
            W.Message("Scripts folder not completely traversed: running out of menus")
            return False
        if done is None:
            done = {}
        if done.has_key(top):
            return True
        done[top] = 1
        import os, string
        try:
            names = os.listdir(top)
        except os.error:
            FrameWork.MenuItem(menu, '(Scripts Folder not found)', None, None)
            return True
        savedir = os.getcwd()
        os.chdir(top)
        for name in names:
            if name == "CVS":
                continue
            try:
                fsr, isdir, isalias = File.FSResolveAliasFile(name, 1)
            except:
                # maybe a broken alias
                continue
            path = fsr.as_pathname()
            if done.has_key(path):
                continue
            name = string.strip(name)
            if os.name == "posix":
                name = unicode(name, "utf-8")
            if name[-3:] == '---':
                menu.addseparator()
            elif isdir:
                submenu = FrameWork.SubMenu(menu, name)
                if not self.scriptswalk(path, submenu, done):
                    return False
            else:
                creator, type = MacOS.GetCreatorAndType(path)
                if type == 'TEXT':
                    if name[-3:] == '.py':
                        name = name[:-3]
                    item = FrameWork.MenuItem(menu, name, None, self.domenu_script)
                    self._scripts[(menu.id, item.item)] = path
            done[path] = 1
        os.chdir(savedir)
        return True

    def domenu_script(self, id, item, window, event):
        (what, message, when, where, modifiers) = event
        path = self._scripts[(id, item)]
        import os
        if not os.path.exists(path):
            self.makescriptsmenu()
            import W
            raise W.AlertError, "File not found."
        if ord(Evt.GetKeys()[7]) & 4:
            self.openscript(path)
        else:
            import W, MacOS, sys
            W.SetCursor("watch")
            sys.argv = [path]
            #cwd = os.getcwd()
            #os.chdir(os.path.dirname(path) + ':')
            try:
                # xxx if there is a script window for this file,
                # exec in that window's namespace.
                # xxx what to do when it's not saved???
                # promt to save?
                if hasattr(MacOS, 'EnableAppswitch'):
                    MacOS.EnableAppswitch(0)
                execfile(path, {'__name__': '__main__', '__file__': path})
            except W.AlertError, detail:
                if hasattr(MacOS, 'EnableAppswitch'):
                    MacOS.EnableAppswitch(-1)
                raise W.AlertError, detail
            except KeyboardInterrupt:
                if hasattr(MacOS, 'EnableAppswitch'):
                    MacOS.EnableAppswitch(-1)
            except:
                if hasattr(MacOS, 'EnableAppswitch'):
                    MacOS.EnableAppswitch(-1)
                import PyEdit
                PyEdit.tracebackwindow.traceback(1)
            else:
                if hasattr(MacOS, 'EnableAppswitch'):
                    MacOS.EnableAppswitch(-1)
            #os.chdir(cwd)

    def openscript(self, filename, lineno=None, charoffset=0, modname=""):
        import os, PyEdit, W
        editor = self.getscript(filename)
        if editor:
            editor.select()
        elif os.path.exists(filename):
            editor = PyEdit.Editor(filename)
        elif filename[-3:] == '.py' or filename[-4:] == '.pyc':
            import imp
            if not modname:
                if filename[-1] == 'c':
                    modname = os.path.basename(filename)[:-4]
                else:
                    modname = os.path.basename(filename)[:-3]
            try:
                # XXX This does not work correctly with packages!
                # XXX The docs say we should do it manually, pack, then sub, then sub2 etc.
                # XXX It says we should use imp.load_module(), but that *reloads* a package,
                # XXX and that's the last thing we want here.
                f, filename, (suff, mode, dummy) = imp.find_module(modname)
            except ImportError:
                raise W.AlertError, "Can't find file for \"%s\"" % modname
            else:
                if not f:
                    raise W.AlertError, "Can't find file for \"%s\"" % modname
                f.close()
            if suff == '.py':
                self.openscript(filename, lineno, charoffset)
                return
            else:
                raise W.AlertError, "Can't find file for \"%s\"" % modname
        else:
            raise W.AlertError, "Can't find file \"%s\"" % filename
        if lineno is not None:
            editor.selectline(lineno, charoffset)
        return editor

    def getscript(self, filename):
        if filename[:1] == '<' and filename[-1:] == '>':
            filename = filename[1:-1]
        import string
        lowpath = string.lower(filename)
        for wid, window in self._windows.items():
            if hasattr(window, "path") and type(window.path) == StringType and \
                            lowpath == string.lower(window.path):
                return window
            elif hasattr(window, "path") and filename == wid.GetWTitle():
                return window

    def getprefs(self):
        import MacPrefs
        return MacPrefs.GetPrefs(self.preffilepath)

    def do_editorprefs(self, *args):
        import PyEdit
        PyEdit.EditorDefaultSettings()

    def do_setwindowfont(self, *args):
        import FontSettings, W
        prefs = self.getprefs()
        settings = FontSettings.FontDialog(prefs.defaultfont)
        if settings:
            prefs.defaultfont, tabsettings = settings
            raise W.AlertError, "Note that changes will only affect new windows!"



class MenuBar(FrameWork.MenuBar):

    possibleIDs = range(10, 256)

    def getnextid(self):
        id = self.possibleIDs[0]
        del self.possibleIDs[0]
        return id

    def __init__(self, parent = None):
        self.bar = MenuToolbox.GetMenuBar()
        MenuToolbox.ClearMenuBar()
        self.menus = {}
        self.parent = parent

    def dispatch(self, id, item, window, event):
        if self.menus.has_key(id):
            self.menus[id].dispatch(id, item, window, event)

    def delmenu(self, id):
        MenuToolbox.DeleteMenu(id)
        if id in self.possibleIDs:
            print "XXX duplicate menu ID!", id
        self.possibleIDs.append(id)


class Menu(FrameWork.Menu):

    def dispatch(self, id, item, window, event):
        title, shortcut, callback, kind = self.items[item-1]
        if type(callback) == StringType:
            callback = self._getmenuhandler(callback)
        if callback:
            import W
            W.CallbackCall(callback, 0, id, item, window, event)

    def _getmenuhandler(self, callback):
        menuhandler = None
        wid = MyFrontWindow()
        if wid and self.bar.parent._windows.has_key(wid):
            window = self.bar.parent._windows[wid]
            if hasattr(window, "domenu_" + callback):
                menuhandler = getattr(window, "domenu_" + callback)
            elif hasattr(self.bar.parent, "domenu_" + callback):
                menuhandler = getattr(self.bar.parent, "domenu_" + callback)
        elif hasattr(self.bar.parent, "domenu_" + callback):
            menuhandler = getattr(self.bar.parent, "domenu_" + callback)
        return menuhandler



import calendar
import unittest

from test import test_support


class CalendarTestCase(unittest.TestCase):
    def test_isleap(self):
        # Make sure that the return is right for a few years, and
        # ensure that the return values are 1 or 0, not just true or
        # false (see SF bug #485794).  Specific additional tests may
        # be appropriate; this tests a single "cycle".
        self.assertEqual(calendar.isleap(2000), 1)
        self.assertEqual(calendar.isleap(2001), 0)
        self.assertEqual(calendar.isleap(2002), 0)
        self.assertEqual(calendar.isleap(2003), 0)

    def test_setfirstweekday(self):
        self.assertRaises(ValueError, calendar.setfirstweekday, 'flabber')
        self.assertRaises(ValueError, calendar.setfirstweekday, -1)
        self.assertRaises(ValueError, calendar.setfirstweekday, 200)
        orig = calendar.firstweekday()
        calendar.setfirstweekday(calendar.SUNDAY)
        self.assertEqual(calendar.firstweekday(), calendar.SUNDAY)
        calendar.setfirstweekday(calendar.MONDAY)
        self.assertEqual(calendar.firstweekday(), calendar.MONDAY)
        calendar.setfirstweekday(orig)

    def test_enumerateweekdays(self):
        self.assertRaises(IndexError, calendar.day_abbr.__getitem__, -10)
        self.assertRaises(IndexError, calendar.day_name.__getitem__, 10)
        self.assertEqual(len([d for d in calendar.day_abbr]), 7)

    def test_days(self):
        for attr in "day_name", "day_abbr":
            value = getattr(calendar, attr)
            self.assertEqual(len(value), 7)
            self.assertEqual(len(value[:]), 7)
            # ensure they're all unique
            d = {}
            for v in value:
                d[v] = 1
            self.assertEqual(len(d), 7)

    def test_months(self):
        for attr in "month_name", "month_abbr":
            value = getattr(calendar, attr)
            self.assertEqual(len(value), 13)
            self.assertEqual(len(value[:]), 13)
            self.assertEqual(value[0], "")
            # ensure they're all unique
            d = {}
            for v in value:
                d[v] = 1
            self.assertEqual(len(d), 13)


class MonthCalendarTestCase(unittest.TestCase):
    def setUp(self):
        self.oldfirstweekday = calendar.firstweekday()
        calendar.setfirstweekday(self.firstweekday)

    def tearDown(self):
        calendar.setfirstweekday(self.oldfirstweekday)

    def check_weeks(self, year, month, weeks):
        cal = calendar.monthcalendar(year, month)
        self.assertEqual(len(cal), len(weeks))
        for i in xrange(len(weeks)):
            self.assertEqual(weeks[i], sum(day != 0 for day in cal[i]))


class MondayTestCase(MonthCalendarTestCase):
    firstweekday = calendar.MONDAY

    def test_february(self):
        # A 28-day february starting of monday (7+7+7+7 days)
        self.check_weeks(1999, 2, (7, 7, 7, 7))

        # A 28-day february starting of tuesday (6+7+7+7+1 days)
        self.check_weeks(2005, 2, (6, 7, 7, 7, 1))

        # A 28-day february starting of sunday (1+7+7+7+6 days)
        self.check_weeks(1987, 2, (1, 7, 7, 7, 6))

        # A 29-day february starting of monday (7+7+7+7+1 days)
        self.check_weeks(1988, 2, (7, 7, 7, 7, 1))

        # A 29-day february starting of tuesday (6+7+7+7+2 days)
        self.check_weeks(1972, 2, (6, 7, 7, 7, 2))

        # A 29-day february starting of sunday (1+7+7+7+7 days)
        self.check_weeks(2004, 2, (1, 7, 7, 7, 7))

    def test_april(self):
        # A 30-day april starting of monday (7+7+7+7+2 days)
        self.check_weeks(1935, 4, (7, 7, 7, 7, 2))

        # A 30-day april starting of tuesday (6+7+7+7+3 days)
        self.check_weeks(1975, 4, (6, 7, 7, 7, 3))

        # A 30-day april starting of sunday (1+7+7+7+7+1 days)
        self.check_weeks(1945, 4, (1, 7, 7, 7, 7, 1))

        # A 30-day april starting of saturday (2+7+7+7+7 days)
        self.check_weeks(1995, 4, (2, 7, 7, 7, 7))

        # A 30-day april starting of friday (3+7+7+7+6 days)
        self.check_weeks(1994, 4, (3, 7, 7, 7, 6))

    def test_december(self):
        # A 31-day december starting of monday (7+7+7+7+3 days)
        self.check_weeks(1980, 12, (7, 7, 7, 7, 3))

        # A 31-day december starting of tuesday (6+7+7+7+4 days)
        self.check_weeks(1987, 12, (6, 7, 7, 7, 4))

        # A 31-day december starting of sunday (1+7+7+7+7+2 days)
        self.check_weeks(1968, 12, (1, 7, 7, 7, 7, 2))

        # A 31-day december starting of thursday (4+7+7+7+6 days)
        self.check_weeks(1988, 12, (4, 7, 7, 7, 6))

        # A 31-day december starting of friday (3+7+7+7+7 days)
        self.check_weeks(2017, 12, (3, 7, 7, 7, 7))

        # A 31-day december starting of saturday (2+7+7+7+7+1 days)
        self.check_weeks(2068, 12, (2, 7, 7, 7, 7, 1))


class SundayTestCase(MonthCalendarTestCase):
    firstweekday = calendar.SUNDAY

    def test_february(self):
        # A 28-day february starting of sunday (7+7+7+7 days)
        self.check_weeks(2009, 2, (7, 7, 7, 7))

        # A 28-day february starting of monday (6+7+7+7+1 days)
        self.check_weeks(1999, 2, (6, 7, 7, 7, 1))

        # A 28-day february starting of saturday (1+7+7+7+6 days)
        self.check_weeks(1997, 2, (1, 7, 7, 7, 6))

        # A 29-day february starting of sunday (7+7+7+7+1 days)
        self.check_weeks(2004, 2, (7, 7, 7, 7, 1))

        # A 29-day february starting of monday (6+7+7+7+2 days)
        self.check_weeks(1960, 2, (6, 7, 7, 7, 2))

        # A 29-day february starting of saturday (1+7+7+7+7 days)
        self.check_weeks(1964, 2, (1, 7, 7, 7, 7))

    def test_april(self):
        # A 30-day april starting of sunday (7+7+7+7+2 days)
        self.check_weeks(1923, 4, (7, 7, 7, 7, 2))

        # A 30-day april starting of monday (6+7+7+7+3 days)
        self.check_weeks(1918, 4, (6, 7, 7, 7, 3))

        # A 30-day april starting of saturday (1+7+7+7+7+1 days)
        self.check_weeks(1950, 4, (1, 7, 7, 7, 7, 1))

        # A 30-day april starting of friday (2+7+7+7+7 days)
        self.check_weeks(1960, 4, (2, 7, 7, 7, 7))

        # A 30-day april starting of thursday (3+7+7+7+6 days)
        self.check_weeks(1909, 4, (3, 7, 7, 7, 6))

    def test_december(self):
        # A 31-day december starting of sunday (7+7+7+7+3 days)
        self.check_weeks(2080, 12, (7, 7, 7, 7, 3))

        # A 31-day december starting of monday (6+7+7+7+4 days)
        self.check_weeks(1941, 12, (6, 7, 7, 7, 4))

        # A 31-day december starting of saturday (1+7+7+7+7+2 days)
        self.check_weeks(1923, 12, (1, 7, 7, 7, 7, 2))

        # A 31-day december starting of wednesday (4+7+7+7+6 days)
        self.check_weeks(1948, 12, (4, 7, 7, 7, 6))

        # A 31-day december starting of thursday (3+7+7+7+7 days)
        self.check_weeks(1927, 12, (3, 7, 7, 7, 7))

        # A 31-day december starting of friday (2+7+7+7+7+1 days)
        self.check_weeks(1995, 12, (2, 7, 7, 7, 7, 1))


def test_main():
    test_support.run_unittest(
        CalendarTestCase,
        MondayTestCase,
        SundayTestCase
    )


if __name__ == "__main__":
    test_main()



"""macgen_info - Generate PYC resource file only"""
import EasyDialogs
import py_resource
from Carbon import Res
import sys

def generate(output, module_dict, debug=0, preload=1):
    fsid = py_resource.create(output)

    for name, module in module_dict.items():
        mtype = module.gettype()
        if mtype not in ['module', 'package']:
            continue
        location = module.__file__

        if location[-4:] == '.pyc':
            # Attempt corresponding .py
            location = location[:-1]
        if location[-3:] != '.py':
            print '*** skipping', location
            continue

        id, name = py_resource.frompyfile(location, name, preload=preload,
                        ispackage=mtype=='package')
        if debug > 0:
            print 'PYC resource %5d\t%s\t%s'%(id, name, location)

    Res.CloseResFile(fsid)

def warnings(module_dict):
    problems = 0
    for name, module in module_dict.items():
        if module.gettype() not in ('builtin', 'module', 'package'):
            problems = problems + 1
            print 'Warning: %s not included: %s %s'%(name, module.gettype(), module)
    return problems



# Build and install an Apple Help Viewer compatible version of the Python
# documentation into the framework.
# Code by Bill Fancher, with some modifications by Jack Jansen.
#
# You must run this as a two-step process
# 1. python setupDocs.py build
# 2. Wait for Apple Help Indexing Tool to finish
# 3. python setupDocs.py install
#
# To do:
# - test whether the docs are available locally before downloading
# - fix buildDocsFromSource
# - Get documentation version from sys.version, fallback to 2.2.1
# - See if we can somehow detect that Apple Help Indexing Tool is finished
# - data_files to setup() doesn't seem the right way to pass the arguments
#
import sys, os, re
from distutils.cmd import Command
from distutils.command.build import build
from distutils.core import setup
from distutils.file_util import copy_file
from distutils.dir_util import copy_tree
from distutils.log import log
from distutils.spawn import spawn
from distutils import sysconfig, dep_util
from distutils.util import change_root
import HelpIndexingTool
import Carbon.File
import time

class DocBuild(build):
    def initialize_options(self):
        build.initialize_options(self)
        self.build_html = None
        self.build_dest = None
        self.download = 0
        self.doc_version = '2.3b1' # Only needed if download is true

    def finalize_options(self):
        build.finalize_options(self)
        if self.build_html is None:
            self.build_html = os.path.join(self.build_base, 'html')
        if self.build_dest is None:
            self.build_dest = os.path.join(self.build_base, 'PythonDocumentation')

    def spawn(self, *args):
        spawn(args, 1,  self.verbose, self.dry_run)

    def downloadDocs(self):
        workdir = os.getcwd()
        url = 'http://www.python.org/ftp/python/doc/%s/html-%s.tgz' % \
                (self.doc_version,self.doc_version)
        os.chdir(self.build_base)
        self.spawn('curl','-O', url)
        os.chdir(workdir)
        tarfile = 'html-%s.tgz' % self.doc_version
## This no longer works due to name changes
##              self.mkpath(self.build_html)
##              os.chdir(self.build_html)
##              self.spawn('tar', '-xzf', '../' + tarfile)
##              os.chdir(workdir)
        print "** Please unpack %s" % os.path.join(self.build_base, tarfile)
        print "** Unpack the files into %s" % self.build_html
        raise RuntimeError, "You need to unpack the docs manually"

    def buildDocsFromSource(self):
        srcdir = '../../..'
        docdir = os.path.join(srcdir, 'Doc')
        htmldir = os.path.join(docdir, 'html')
        spawn(('make','--directory', docdir, 'html'), 1, self.verbose, self.dry_run)
        self.mkpath(self.build_html)
        copy_tree(htmldir, self.build_html)

    def ensureHtml(self):
        if not os.path.exists(self.build_html):
            if self.download:
                self.downloadDocs()
            else:
                self.buildDocsFromSource()

    def hackIndex(self):
        ind_html = 'index.html'
        #print 'self.build_dest =', self.build_dest
        hackedIndex = file(os.path.join(self.build_dest, ind_html),'w')
        origIndex = file(os.path.join(self.build_html,ind_html))
        r = re.compile('<style type="text/css">.*</style>', re.DOTALL)
        hackedIndex.write(r.sub('<META NAME="AppleTitle" CONTENT="Python Documentation">',origIndex.read()))

    def hackFile(self,d,f):
        origPath = os.path.join(d,f)
        assert(origPath[:len(self.build_html)] == self.build_html)
        outPath = os.path.join(self.build_dest, d[len(self.build_html)+1:], f)
        (name, ext) = os.path.splitext(f)
        if os.path.isdir(origPath):
            self.mkpath(outPath)
        elif ext == '.html':
            if self.verbose: print 'hacking %s to %s' % (origPath,outPath)
            hackedFile = file(outPath, 'w')
            origFile = file(origPath,'r')
            hackedFile.write(self.r.sub('<dl><dt><dd>', origFile.read()))
        else:
            copy_file(origPath, outPath)

    def hackHtml(self):
        self.r = re.compile('<dl><dd>')
        os.path.walk(self.build_html, self.visit, None)

    def visit(self, dummy, dirname, filenames):
        for f in filenames:
            self.hackFile(dirname, f)

    def makeHelpIndex(self):
        app = '/Developer/Applications/Apple Help Indexing Tool.app'
        self.spawn('open', '-a', app , self.build_dest)
        print "Please wait until Apple Help Indexing Tool finishes before installing"

    def makeHelpIndex(self):
        app = HelpIndexingTool.HelpIndexingTool(start=1)
        app.open(Carbon.File.FSSpec(self.build_dest))
        sys.stderr.write("Waiting for Help Indexing Tool to start...")
        while 1:
            # This is bad design in the suite generation code!
            idle = app._get(HelpIndexingTool.Help_Indexing_Tool_Suite._Prop_idleStatus())
            time.sleep(10)
            if not idle: break
            sys.stderr.write(".")
        sys.stderr.write("\n")
        sys.stderr.write("Waiting for Help Indexing Tool to finish...")
        while 1:
            # This is bad design in the suite generation code!
            idle = app._get(HelpIndexingTool.Help_Indexing_Tool_Suite._Prop_idleStatus())
            time.sleep(10)
            if idle: break
            sys.stderr.write(".")
        sys.stderr.write("\n")


    def run(self):
        self.ensure_finalized()
        self.mkpath(self.build_base)
        self.ensureHtml()
        if not os.path.isdir(self.build_html):
            raise RuntimeError, \
            "Can't find source folder for documentation."
        self.mkpath(self.build_dest)
        if dep_util.newer(os.path.join(self.build_html,'index.html'), os.path.join(self.build_dest,'index.html')):
            self.mkpath(self.build_dest)
            self.hackHtml()
            self.hackIndex()
            self.makeHelpIndex()

class AHVDocInstall(Command):
    description = "install Apple Help Viewer html files"
    user_options = [('install-doc=', 'd',
            'directory to install HTML tree'),
             ('root=', None,
             "install everything relative to this alternate root directory"),
            ]

    def initialize_options(self):
        self.build_dest = None
        self.install_doc = None
        self.prefix = None
        self.root = None

    def finalize_options(self):
        self.set_undefined_options('install',
                ('prefix', 'prefix'),
                ('root', 'root'))
#               import pdb ; pdb.set_trace()
        build_cmd = self.get_finalized_command('build')
        if self.build_dest == None:
            build_cmd = self.get_finalized_command('build')
            self.build_dest = build_cmd.build_dest
        if self.install_doc == None:
            self.install_doc = os.path.join(self.prefix, 'Resources/Python.app/Contents/Resources/English.lproj/PythonDocumentation')
        print 'INSTALL', self.build_dest, '->', self.install_doc

    def run(self):
        self.finalize_options()
        self.ensure_finalized()
        print "Running Installer"
        instloc = self.install_doc
        if self.root:
            instloc = change_root(self.root, instloc)
        self.mkpath(instloc)
        copy_tree(self.build_dest, instloc)
        print "Installation complete"

def mungeVersion(infile, outfile):
    i = file(infile,'r')
    o = file(outfile,'w')
    o.write(re.sub('\$\(VERSION\)',sysconfig.get_config_var('VERSION'),i.read()))
    i.close()
    o.close()

def main():
    # turn off warnings when deprecated modules are imported
##      import warnings
##      warnings.filterwarnings("ignore",category=DeprecationWarning)
    setup(name = 'Documentation',
            version = '%d.%d' % sys.version_info[:2],
            cmdclass = {'install_data':AHVDocInstall, 'build':DocBuild},
            data_files = ['dummy'],
            )

if __name__ == '__main__':
    main()



# IBCarbonscan.py

import sys
import os
import string

from bgenlocations import TOOLBOXDIR, BGENDIR
sys.path.append(BGENDIR)

from scantools import Scanner_OSX

def main():
    print "---Scanning IBCarbonRuntime.h---"
    input = ["IBCarbonRuntime.h"]
    output = "IBCarbongen.py"
    defsoutput = TOOLBOXDIR + "IBCarbonRuntime.py"
    scanner = IBCarbon_Scanner(input, output, defsoutput)
    scanner.scan()
    scanner.close()
    print "=== Testing definitions output code ==="
    execfile(defsoutput, {}, {})
    print "--done scanning, importing--"
    import IBCarbonsupport
    print "done"

class IBCarbon_Scanner(Scanner_OSX):

    def destination(self, type, name, arglist):
        classname = "IBCarbonFunction"
        listname = "functions"
        if arglist:
            t, n, m = arglist[0]
            if t == "IBNibRef" and m == "InMode":
                classname = "IBCarbonMethod"
                listname = "methods"
        return classname, listname

    def makeblacklistnames(self):
        return [
                "DisposeNibReference",                          # taken care of by destructor
                "CreateNibReferenceWithCFBundle",  ## need to wrap CFBundle.h properly first
                ]

    def makerepairinstructions(self):
        return []


if __name__ == "__main__":
    main()



# copyright 1996-2001 Just van Rossum, Letterror. just@letterror.com

# keep this (__main__) as clean as possible, since we are using
# it like the "normal" interpreter.

__version__ = '1.0.2'
import sys
import os

def init():
    import MacOS
    if hasattr(MacOS, 'EnableAppswitch'):
        MacOS.EnableAppswitch(-1)

    try:
        import autoGIL
    except ImportError:
        pass
    else:
        autoGIL.installAutoGIL()

    from Carbon import Qd, QuickDraw
    Qd.SetCursor(Qd.GetCursor(QuickDraw.watchCursor).data)

    import macresource
    import sys, os
    macresource.need('DITL', 468, "PythonIDE.rsrc")
    widgetrespathsegs = [sys.exec_prefix, "Mac", "Tools", "IDE", "Widgets.rsrc"]
    widgetresfile = os.path.join(*widgetrespathsegs)
    if not os.path.exists(widgetresfile):
        widgetrespathsegs = [os.pardir, "Tools", "IDE", "Widgets.rsrc"]
        widgetresfile = os.path.join(*widgetrespathsegs)
    refno = macresource.need('CURS', 468, widgetresfile)
    if os.environ.has_key('PYTHONIDEPATH'):
        # For development set this environment variable
        ide_path = os.environ['PYTHONIDEPATH']
    elif refno:
        # We're not a fullblown application
        idepathsegs = [sys.exec_prefix, "Mac", "Tools", "IDE"]
        ide_path = os.path.join(*idepathsegs)
        if not os.path.exists(ide_path):
            idepathsegs = [os.pardir, "Tools", "IDE"]
            for p in sys.path:
                ide_path = os.path.join(*([p]+idepathsegs))
                if os.path.exists(ide_path):
                    break

    else:
        # We are a fully frozen application
        ide_path = sys.argv[0]
    if ide_path not in sys.path:
        sys.path.insert(1, ide_path)


init()
del init

import PythonIDEMain as _PythonIDEMain
_PythonIDEMain.PythonIDE()



import string
import sys
import traceback


try:
    sys.ps1
except AttributeError:
    sys.ps1 = ">>> "
try:
    sys.ps2
except AttributeError:
    sys.ps2 = "... "


def print_exc(limit=None, file=None):
    if not file:
        file = sys.stderr
    # we're going to skip the outermost traceback object, we don't
    # want people to see the line which excecuted their code.
    tb = sys.exc_traceback
    if tb:
        tb = tb.tb_next
    try:
        sys.last_type = sys.exc_type
        sys.last_value = sys.exc_value
        sys.last_traceback = tb
        traceback.print_exception(sys.last_type, sys.last_value,
                                sys.last_traceback, limit, file)
    except:
        print '--- hola! ---'
        traceback.print_exception(sys.exc_type, sys.exc_value,
                                sys.exc_traceback, limit, file)


class PyInteractive:

    def __init__(self):
        import codeop
        self._pybuf = ""
        self._compile = codeop.Compile()

    def executeline(self, stuff, out = None, env = None):
        if env is None:
            import __main__
            env = __main__.__dict__
        if out:
            saveerr, saveout = sys.stderr, sys.stdout
            sys.stderr = sys.stdout = out
        try:
            if self._pybuf:
                self._pybuf = self._pybuf + '\n' + stuff
            else:
                self._pybuf = stuff

            # Compile three times: as is, with \n, and with \n\n appended.
            # If it compiles as is, it's complete.  If it compiles with
            # one \n appended, we expect more.  If it doesn't compile
            # either way, we compare the error we get when compiling with
            # \n or \n\n appended.  If the errors are the same, the code
            # is broken.  But if the errors are different, we expect more.
            # Not intuitive; not even guaranteed to hold in future
            # releases; but this matches the compiler's behavior in Python
            # 1.4 and 1.5.
            err = err1 = err2 = None
            code = code1 = code2 = None

            # quickly get out of here when the line is 'empty' or is a comment
            stripped = string.strip(self._pybuf)
            if not stripped or stripped[0] == '#':
                self._pybuf = ''
                sys.stdout.write(sys.ps1)
                sys.stdout.flush()
                return

            try:
                code = self._compile(self._pybuf, "<input>", "single")
            except SyntaxError, err:
                pass
            except:
                # OverflowError. More?
                print_exc()
                self._pybuf = ""
                sys.stdout.write(sys.ps1)
                sys.stdout.flush()
                return

            try:
                code1 = self._compile(self._pybuf + "\n", "<input>", "single")
            except SyntaxError, err1:
                pass

            try:
                code2 = self._compile(self._pybuf + "\n\n", "<input>", "single")
            except SyntaxError, err2:
                pass

            if code:
                try:
                    exec code in env
                except:
                    print_exc()
                self._pybuf = ""
            elif code1:
                pass
            elif err1 == err2 or (not stuff and self._pybuf):
                print_exc()
                self._pybuf = ""
            if self._pybuf:
                sys.stdout.write(sys.ps2)
                sys.stdout.flush()
            else:
                sys.stdout.write(sys.ps1)
                sys.stdout.flush()
        finally:
            if out:
                sys.stderr, sys.stdout = saveerr, saveout
